--
-- This is a generalized piece of code for surrounding an actor
-- with eight divided pieces of image aligned to the corners. It is
-- intended to be added as a child of the host label.
--

--
-- The fitting algorithm is:
--
-- Center corner images on the corners of the host object.
-- If there's any room for edge texture, stretch edge textures between the 
-- edges of the corner textures, centering them on the sides of the host
-- object.
--

--Usage note: There seems to be a graphical glitch in stretching the border
--segments. It's best to have border segments bigger than the object could
--possibly get.

--children:

--topLeftCorner
--topRightCorner
--bottomLeftCorner
--bottomRightCorner
--topEdge
--bottomEdge
--leftEdge
--rightEdge

--hostObject

Resources.BubbleBorder = Resources.Actor:new {
	resource = "Actor",
	alpha = 0,
	zorder = -1,
	
	topLeftCorner = Resources.Actor:new {
		resource = "Actor",
		alignment = "Center",
		alignTo = "TopLeft",
	},
	topRightCorner = Resources.Actor:new {
		resource = "Actor",
		alignment = "Center",
		alignTo = "TopRight",
	},
	bottomRightCorner = Resources.Actor:new {
		resource = "Actor",
		alignment = "Center",
		alignTo = "BottomRight",
	},
	bottomLeftCorner = Resources.Actor:new {
		resource = "Actor",
		alignment = "Center",
		alignTo = "BottomLeft",
	},
	topEdge = Resources.Actor:new {
		resource = "Actor",
		alignment = "Center",
		alignTo = "MidTop",
	},
	bottomEdge = Resources.Actor:new {
		resource = "Actor",
		alignment = "Center",
		alignTo = "MidBottom",
	},
	leftEdge = Resources.Actor:new {
		resource = "Actor",
		alignment = "Center",
		alignTo = "MidLeft",
	},
	rightEdge = Resources.Actor:new {
		resource = "Actor",
		alignment = "Center",
		alignTo = "MidRight",
	},
}

function ResizeHandler(eventData)
	if eventData == nil then return end
	
	local self = eventData.receiver
	local source = eventData.sender
	if(self == source) then
		ResizeBorder(self)
	end
end

function ResizeBorder(hostObject)
	local border = hostObject.Border
	
	--Determine minimum size we can shrink the border to
	
	local minWidth = (border.topRightCorner.width + border.topLeftCorner.width)/2
	local minHeight = (border.topRightCorner.height + border.bottomRightCorner.height)/2
	
	local borderSize = 	hostObject.size
	if(borderSize.x < minWidth) then
		borderSize.x = minWidth
	end
	if(borderSize.y < minHeight) then
		borderSize.y = minHeight
	end
	
	border.size = borderSize
	
	--Resize the edges to suit
	local newWidth = borderSize.x - minWidth
	local newHeight = borderSize.y - minHeight
	
	if(newWidth % 2 == 1) then
		newWidth = newWidth + 1
	end
	if(newHeight % 2 == 1) then
		newHeight = newHeight + 1
	end
	
	border.topEdge.width = newWidth
	border.bottomEdge.width = newWidth
	border.leftEdge.height = newHeight
	border.rightEdge.height = newHeight
end

--BorderProps is indexed by the name of the border component to the
--properties to apply, "root" which applies to the border itself, 
--and "all" which applies to all components

function MakeBubbleBorder(hostObject,borderProps)
	--Make the edges
	hostObject.Border = Resources.BubbleBorder:new {
		resource = "BubbleBorder",
	}
	local border = hostObject.Border
	local components = {
		"topLeftCorner",
		"topRightCorner",
		"bottomLeftCorner",
		"bottomRightCorner",
		"topEdge",
		"bottomEdge",
		"leftEdge",
		"rightEdge",
	}
	
	for index,componentName in ipairs(components) do
		local specificProps  = borderProps[componentName]
		local allProps = borderProps["all"]
		if(allProps ~= nil) then
			border[componentName]:ApplyProperties(allProps)
		end
		if(specificProps ~= nil) then
			border[componentName]:ApplyProperties(specificProps)
		end
	end
	if(borderProps["root"] ~= nil) then
		border:ApplyProperties(borderProps["root"])
	end
	
	hostObject.GURU_ACTOR_RESIZED = ResizeHandler
	hostObject:AddObserver(EventMap.GURU_ACTOR_RESIZED,hostObject)
	
	ResizeBorder(hostObject)
end