-- Searches the target for a child with name
-- If target is nil CurrentScreen is searched
-- If recursive is nil children will be searched by default
function FindActor(name, target, recursive)
	local target = target or CurrentScreen()
	local recursive = recursive or true
	local child = target:FindChild(name, recursive)
	if child ~= nil then 
		logprint(child.name.." found in "..target.name.."\n")
	else
		logprint("Failed to find "..name.." in "..target.name.."\n") 
	end
	return child
end

function FadeOut(target, time)
	assert(target, "FadeOut() was passed a nil `target`.")
	assert(time, "FadeOut() was passed a nil `time`.")
	assert(type(time) == "number", "FadeIn() was passed a non int `time`."..type(time))
	target:FadeOut(time, false, true)
end

function FadeOutNoChildren(target, time)
	assert(target, "FadeOutNoChildren() was passed a nil `target`.")
	assert(time, "FadeOutNoChildren() was passed a nil `time`.")
	assert(type(time) == "number", "FadeOutNoChildren() was passed a non int `time`.")
	target:FadeOut(time, false, false)
end

function FadeIn(target, time)
	assert(target, "FadeIn() was passed a nil `target`.")
	assert(time, "FadeIn() was passed a nil `time`.")
	assert(type(time) == "number", "FadeIn() was passed a non int `time`."..type(time))
	target:FadeIn(time, true)
end

function FadeInNoChildren(target, time)
	assert(target, "FadeInNoChildren() was passed a nil `target`.")
	assert(time, "FadeInNoChildren() was passed a nil `time`.")
	assert(type(time) == "number", "FadeInNoChildren() was passed a non int `time`.")
	target:FadeIn(time, false)
end

function FadeOutAndDestroy(target, time)
	assert(target, "FadeOut() was passed a nil `target`.")
	assert(time, "FadeOut() was passed a nil `time`.")
	assert(type(time) == "number", "FadeIn() was passed a non int `time`.")
	target:FadeOut(time, true, true)
end

function StopFade(target)
	assert(target, "StopFade() was passed a nil `target`.")
	target:StopFade()
end

-- Slides an Actor and pauses the current script until it finishes.
function SlideAndWait(target, dest, duration)
	assert(Script.current, "SlideAndWait() must be called from within a running script.")
	assert(target, "SlideAndWait() was passed a nil `target`.")
	assert(dest, "SlideAndWait() was passed a nil `dest`.")
	assert(duration, "SlideAndWait() was passed a nil `duration`.")

	target:SlideTo(dest, duration)
	ScriptDelay(duration+32,"DialogTimers")
	--JSUPNIK Note: This is a hack to deal with desynchronization that can happen between the clouds
	--and the hiding clouds afterwards in Slingo Supreme Iphone's main menu, if you change screens.
	--See Fogbugz 10551 and 10571 for more information.
	while(target:IsSliding()) do
		ScriptBreak()
	end
end

-- Slides an Actor off screen in 4 directions - bottom,top,left,right - assumes screen is 800x600
function SlideOff(target, duration, direction, wait)
	assert(Script.current, "SlideOff() must be called from within a running script.")
	assert(target, "SlideOff() was passed a nil `target`.")
	assert(duration, "SlideOff() was passed a nil `duration`.")
	assert(direction, "SlideOff() was passed a nil `direction`.")
	-- NOTE: the 'wait' parameter can be nil, in which case, this function will not wait
	
	--logprint("******************************")
	--logprint("SlideOff:"..target.name)
	
	local currentPos	= target.position
	local destPos		= target.position
	
	--logprint("SlideOff:currentPos "..currentPos.x.." "..currentPos.y)
	--logprint("SlideOff:direction "..direction)
	--logprint("SlideOff:target.size.x "..target.size.x)
	--logprint("SlideOff:target.size.y "..target.size.y)
	
	if (direction == "bottom") then
		destPos.y = target:GetTreeRoot().height
	elseif (direction == "top") then
		destPos.y = -target.size.y
	elseif (direction == "left") then
		destPos.x = -target.size.x
	elseif (direction == "right") then
		destPos.x = target:GetTreeRoot().width
	end
	
	--logprint("SlideOff:destPos "..destPos.x.." "..destPos.y)
	
	target:SlideTo(destPos, duration)
	
	if (wait == "wait") then
		ScriptDelay(duration,"DialogTimers")
		-- make invisible but reset position to default
		target.visible = false
		target.position = currentPos
	end
end

-- Slides an Actor on to screen from another one
function SlideFrom(target, fromPosition, duration, wait)
	assert(Script.current, "SlideFrom() must be called from within a running script.")
	assert(target, "SlideFrom() was passed a nil `target`.")
	assert(duration, "SlideFrom() was passed a nil `duration`.")
	-- NOTE: the 'wait' parameter can be nil, in which case, this function will not wait
	
	local currentPos	= target.position
	local startPos		= fromPosition
	
	target.position = startPos
	
	target.visible = true
	target:SlideTo(currentPos, duration)
	
	if (wait == "wait") then
		ScriptDelay(duration,"DialogTimers")
	end
end

-- Slides an Actor on to screen in 4 directions - up,down,left,right
function SlideOn(target, duration, direction, wait, offset)
	assert(Script.current, "SlideOn() must be called from within a running script.")
	assert(target, "SlideOn() was passed a nil `target`.")
	assert(duration, "SlideOn() was passed a nil `duration`.")
	assert(direction, "SlideOn() was passed a nil `direction`.")
	-- NOTE: the 'wait' parameter can be nil, in which case, this function will not wait
	
	local currentPos	= target.position
	local startPos		= target.position
	
	if (direction == "bottom") then
		startPos.y = target:GetTreeRoot().height
	elseif (direction == "top") then
		startPos.y = -target.size.y
	elseif (direction == "left") then
		startPos.x = -target.size.x
	elseif (direction == "right") then
		startPos.x = target:GetTreeRoot().width
	end
	
	if offset then
		offset = PointF_New(offset)
		startPos.x = startPos.x + offset.x
		startPos.y = startPos.y + offset.y
	end
	
	target.position = startPos
	
	target.visible = true
	target:SlideTo(currentPos, duration)
	
	if (wait == "wait") then
		ScriptDelay(duration,"DialogTimers")
	end
end

-- Waits for a single Actor, or a set of Actors to finish sliding.
-- It is expected that the Actor will already have begun sliding.
function WaitForSlideToFinish(target)
	-- This function operates internally on a table of actors.
	local actors
	
	-- If `target` is a single actor, place it into a table.  Otherwise, assume it is a table and
	-- make a copy of it to work on.
	if type(target) == "userdata" then
		actors = { target }
	elseif type(target) == "table" then
		actors = DeepCopy(target)
	else
		assert(false, "`target` is neither an Actor nor a table.")
	end
	
	-- Wait for each actor to finish sliding.
	while true do
		-- Cycle through each actor in actors, removing ones that aren't sliding.
		local idx = 1
		while idx <= table.getn(actors) do
			if actors[idx]:IsSliding() == false then
				table.remove(actors, idx)
			else
				idx = idx + 1
			end
		end
		
		-- All actors have been inspected.  If none are left (true when none are sliding), exit
		-- the script.  Otherwise, break execution until the next tick.
		if table.getn(actors) == 0 then
			return
		else
			ScriptBreak()
		end
	end
end

-- Slides an Actor at a current speed and pauses the current script until it finishes.
function SlideAtSpeed(target, dest, pixelsPerSecond)
	assert(Script.current, "SlideAtSpeed() must be called from within a running script.")
	assert(target, "SlideAtSpeed() was passed a nil `target`.")
	assert(dest, "SlideAtSpeed() was passed a nil `dest`.")
	assert(pixelsPerSecond, "SlideAtSpeed() was passed a nil `pixelsPerSecond`.")

	local position = target.position
	local x = position.x - (dest.x or dest[1])
	local y = position.y - (dest.y or dest[2])
	local length = math.sqrt((x*x) + (y*y))
	local duration = (length / pixelsPerSecond) * 1000.0
	
	--logprint("SlideAtSpeed: position.x "..position.x.." position.y "..position.y)
	--logprint("SlideAtSpeed: dest.x "..dest.x.." dest.y "..dest.y)
	--logprint("SlideAtSpeed: pixelsPerSecond "..pixelsPerSecond.." length "..length.." duration "..duration)
	
	target:SlideTo(dest, duration)
end

-- 'Named' version of SlideAtSpeed (for the C++ 'Script' object)
Scripts.SlideAtSpeed = SlideAtSpeed

-- Slides an Actor on to screen in 4 directions - up,down,left,right - assumes screen is 800x600
function SlideOnAtSpeed(target, pixelsPerSecond, direction, wait)
	assert(Script.current, "SlideOnAtSpeed() must be called from within a running script.")
	assert(target, "SlideOnAtSpeed() was passed a nil `target`.")
	assert(pixelsPerSecond, "SlideOnAtSpeed() was passed a nil `pixelsPerSecond`.")
	assert(direction, "SlideOnAtSpeed() was passed a nil `direction`.")
	-- NOTE: the 'wait' parameter can be nil, in which case, this function will not wait
	
	local currentPos	= target.position
	local startPos		= target.position
	
	if (direction == "bottom") then
		startPos.y = target:GetTreeRoot().height
	elseif (direction == "top") then
		startPos.y = -target.size.y
	elseif (direction == "left") then
		startPos.x = -target.size.x
	elseif (direction == "right") then
		startPos.x = target:GetTreeRoot().width
	end
	
	target.position = startPos
	
	target.visible = true
	
	local x = currentPos.x - (startPos.x)
	local y = currentPos.y - (startPos.y)
	local length = math.sqrt((x*x) + (y*y))
	local duration = (length / pixelsPerSecond) * 1000.0
	
	SlideAtSpeed(target,currentPos, pixelsPerSecond)
	
	if (wait == "wait") then
		ScriptDelay(duration,"DialogTimers")
	end
end

--[[
	GetGlobalAlignmentPoint

	Retrieves a global coordinate that can be used to align actors.

	Params:
		params: A table of parameters.  See the "Named Params" section for more info.
		
	Named Params:
		alignmentTarget: An instance of Actor which, if specified, will cause the returned
			alignment point to be positioned relative to the actor's global rect.
		alignmentPoint: Specifies where in the alignmentTarget that the returned alignment point
			should be positioned.  Defaults to "Center".
		offset: Offsets the position of the returned alignment point by a given amount.
		position: Specifies a default alignment point.  This value will be ignored if
			alignmentTarget is set.
	
	Return Values:
		1: A point with X and Y coordinates, represented as integers at table indices 1 and 2.
		2: An error message, or nil if no errors occurred.
--]]

function GetGlobalAlignmentPoint(params)

	-- Extract params, using a reasonable set of defaults.
	params = params or {}
	local alignmentPoint = params.alignmentPoint or "Center"
	local alignmentTarget = params.alignmentTarget or nil
	local offset = params.offset or {0, 0}
	local position = params.position or nil

	local limitToVisible = true
	if params.limitToVisible ~= nil then
		limitToVisible = params.limitToVisible
	end

	-- Assume the actor is on the current screen.
	local screen = CurrentScreen()

	-- Make sure that 'alignmentTarget' is either nil or an actor.  If it's something else, then
	-- try to convert it to an actor.
	if type(alignmentTarget) == "string" then
		alignmentTarget = { visibleChild = alignmentTarget }
	end
	
	if type(alignmentTarget) == "table" then
		if type(alignmentTarget.tag) == "string" then
			local tag = alignmentTarget.tag
			alignmentTarget = TagManager_RetrieveActor(tag, screen, limitToVisible)
			if not alignmentTarget then
				return nil, "alignmentTarget not found; specified tag = '"..tostring(tag).."'"
			end
		elseif type(alignmentTarget.visibleChild) == "string" then
			local visibleChild = alignmentTarget.visibleChild
			alignmentTarget = screen:FindVisibleChild(visibleChild, true)
			if not alignmentTarget then
				return nil, "alignmentTarget not found; specified visibleChild = '"..tostring(visibleChild).."'"
			end
		end
		
		if not alignmentTarget then
			return nil, "alignmentTarget not found; alignmentTarget was specified without search parameters"
		end
	end
	
	-- Calculate 'position' if 'alignmentTarget' is set.
	if type(alignmentTarget) == 'userdata' then
		-- Assume 'alignmentTarget' is an instance of Actor
		local targetRect = alignmentTarget:GlobalRect()
		position = AlignablePoint_GetPoint(alignmentPoint, targetRect)
	elseif type(alignmentTarget) ~= 'nil' then
		error(string.format("function was passed an invalid alignmentTarget (of type, \"%s\")",
			type(alignmentTarget)))
	end
	
	-- Apply a default to 'position', if need be.
	if not position then
		position = {}
		position.x = screen.position.x + (screen.size.x / 2)
		position.y = screen.position.y + (screen.size.y / 2)
	end

	-- Add 'offset' to 'position'.
	if offset then
		offset = PointF_New(offset)
		position = PointF_New(position)
		position.x = position.x + offset.x
		position.y = position.y + offset.y
	end
	
	return position
end


--[[
	CreateAndAttachScript

	Backend for AddScriptWithParams and RunScriptWithParams
--]]

local function CreateAndAttachScript(methodName, input)
	if table.getn(input) == 0 then
		error("invalid number of args passed to AddScriptWithParams or RunScriptWithParams")
	end

	local target, scriptFunction, params, tags
	local targetIsFirstParam = true
	if type(input[1]) == "table" then
		target = input[1].target
		scriptFunction = input[1].script
		params = input[1].params or {}
		tags = input[1].tags
		
		if input[1].targetIsFirstParam ~= nil then
			targetIsFirstParam = input[1].targetIsFirstParam
		end
		
		if type(tags) == "string" then
			tags = { tags }
		end
	else
		target = input[1]
		scriptFunction = input[2]

		params = {}
		for i=3,table.getn(input) do
			table.insert(params, input[i])
		end
	end
	
	if not target then
		error("an invalid target actor was passed to AddScriptWithParams or RunScriptWithParams")
	end
	if not scriptFunction then
		error("an invalid script was passed to AddScriptWithParams or RunScriptWithParams")
	end
	if tags and type(tags) ~= "table" then
		error("an invalid set of tags was passed to AddScriptWithParams or RunScriptWithParams: 'tags' must be a table, a string, or be nil")
	end

	local scriptObject = CreateScript(scriptFunction)
	
	if type(scriptFunction) == "function" then
		local info = debug.getinfo(scriptFunction, "Sn")
		
		scriptObject.name = string.format("Script [\"%s\":%s:%d]",
			info.name or "",
			info.short_src,
			info.linedefined)
	end

	if targetIsFirstParam then
		table.insert(params, 1, target)
	end
	scriptObject:SetParams(params)

	if tags then
		scriptObject:AddTags(tags)
	end

	target[methodName](target, scriptObject)
end



--[[
	AddScriptWithParams

	Adds a script to an actor, along with a set of parameters.
	
	Params (Option #1):
		targetActor: The actor to attach the script to.
		scriptFunction: A Lua function, or a string that references a function in the global
			table, "Scripts".
		...: [Optional] All additional parameters will be passed into the script's function.
		
	Params (Option #2):
		input: A table of params with the following keys:
			target: The actor to attach the script to.
			script: A Lua function, or a string that references a function in the global table,
				"Scripts".
			params: [OPTIONAL] An array of parameters to be unpacked and passed into the script as
				a set of parameters.
			tags: [OPTIONAL] An array of strings, or a single string, which will be attached to the
				script as tags.
	
	Return Values:
		None
		
	Notes:
		The 1st parameter passed into the script function will be the target actor.
--]]
	
function AddScriptWithParams(...)
	CreateAndAttachScript("AddScript", arg)
end

--[[
	RunScriptWithParams

	Adds a script to an actor, along with a set of parameters, then ticks the script once.
	
	Params (Option #1):
		targetActor: The actor to attach the script to.
		scriptFunction: A Lua function, or a string that references a function in the global
			table, "Scripts".
		...: [Optional] All additional parameters will be passed into the script's function.
		
	Params (Option #2):
		input: A table of params with the following keys:
			target: The actor to attach the script to.
			script: A Lua function, or a string that references a function in the global table,
				"Scripts".
			params: [OPTIONAL] An array of parameters to be unpacked and passed into the script as
				a set of parameters.
			tags: [OPTIONAL] An array of strings, or a single string, which will be attached to the
				script as tags.
	
	Return Values:
		None
		
	Notes:
		The 1st parameter passed into the script function will be the target actor.
--]]

function RunScriptWithParams(...)
	CreateAndAttachScript("RunScript", arg)
end


--[[
	WaitForTaggedScriptCompletion

	Waits for all of an actor's attached scripts with a given tag to end.

	Params:
		targetActor: An instance of Actor.
		tagName: A string containing the name of a script, or a tag attached to
			a script.
	
	Return Values:
		none
--]]
function WaitForTaggedScriptCompletion(targetActor, tagName)
	if not targetActor then
		error("1st param, 'targetActor', is nil")
	end
	if type(targetActor) ~= "userdata" then
		error("1st param, 'targetActor', is not an actor")
	end

	while targetActor:HasTaggedScript(tagName) do
		ScriptBreak()
	end
end


--[[
	GetActorsWithTaggedScript

	Recursively among an actor and its children for any actor with an
	attached script containing a given tag.

	Params:
		baseActor: An actor.
		tagName: A string containing the name of a script, or a tag attached to
			a script.
	
	Return Values:
		An array of actors that contain a script with the specified tag.
--]]
function GetActorsWithTaggedScript(baseActor, tagName)
	local results = {}

	local iter = baseActor:CreateTreeIterator()
	while iter:IsValid() do
		local child = iter:GetValue()
		if child:HasTaggedScript(tagName) then
			table.insert(results, child)
		end
		iter:Advance()
	end
	DeleteObject(iter)

	return results
end



--[[
	Class: Actor
	Description
	Parent Class: Object
	Description: Base Class for all screen widgets
--]]

Resources.Actor = Resources.Object:new 
{
	class = "Actor",	-- Must be same as Template Name
	--Color = "Black", --This would change the actor's draw type to colored. Unfortunately, making this the default in the engine causes black screens in various places that would require chasing down. For now, leaving it commented out, and fixing anything that looks broken.
}

--[[
	Class: ActorGroup
	Parent Class: Object
	Properties:
		All Actor Properties
	
--]]
Resources.ActorGroup = Resources.Object:new
{
	class 	= "ActorGroup",
}
