
--[[
    Edits an actor, as declared in the global, Lua table, Resources[]

    Example:
        -- edits the resource, Resources.SomeDialog
        Edit("SomeDialog")

--]]
function Edit(actorResource)
    if not SCREEN_MGR then
        error("SCREEN_MGR is not available")
    end
    local oldScreen = SCREEN_MGR:CurrentScreen()
    local editorScreen = SCREEN_MGR:GetScreen("ActorEditorScreen")
    if (not oldScreen) or (oldScreen ~= editorScreen) then
        SCREEN_MGR:SwitchToScreen(editorScreen)
    end
    editorScreen.RootActor:Edit(actorResource, oldScreen)
end

--[[
    Stops editing an actor, and switches back to the previous screen

    Example:
        EditStop()
--]]
function EditStop()
    if not SCREEN_MGR then
        error("SCREEN_MGR is not available")
    end
    local oldScreen = SCREEN_MGR:CurrentScreen()
    local editorScreen = SCREEN_MGR:GetScreen("ActorEditorScreen")
    ActorEditorScreen_actorResource = nil
    if oldScreen == editorScreen then
        local nextScreen = SCREEN_MGR:GetScreen(ActorEditorScreen_prevScreen)
        if not nextScreen then
            error(string.format("Can't switch to previous screen, which is '%s'", tostring(ActorEditorScreen_prevScreen)))
        else
            SCREEN_MGR:SwitchToScreen(nextScreen)
        end
    end
end

--[[
    Sets the background image 

    Examples:
        -- Sets the background image to "Foo.jpg":
        EditSetBackground("Foo.jpg")

        -- Clear any previously-set background
        EditSetBackground()
--]]
function EditSetBackground(backgroundName)
    if not SCREEN_MGR then
        error("SCREEN_MGR is not available")
    end
    ActorEditorScreen_background = backgroundName
    local editorScreen = SCREEN_MGR:GetScreen("ActorEditorScreen")
    if editorScreen then
        editorScreen.RootActor:RecreateBackground()
    end
end

--[[
    ActorEditorScreen

    A GuruEngine Screen, used in conjunction with Edit*() commands, to edit arbitrary, Lua-defined Actor-resources.

    This screen uses a few Lua globals to track state across .lua-file reloads.  Among them are:
     * ActorEditorScreen_actorResource -- the name of the resource to edit; gets set by the Edit() command
     * ActorEditorScreen_prevScreen    -- the previous screen; will be used if EditStop() is invoked; gets set by the Edit() command
     * ActorEditorScreen_background    -- the name of a background image to use, or nil; gets set by the EditSetBackground() command
--]]
Resources.ActorEditorScreen = Resources.BuilderScreen:new {
    resource = "BuilderScreen",

    OnPostSpawn = function (self)
        if SCREEN_MGR and SCREEN_MGR:CurrentScreen() == self.screen then
            self:RecreateBackground()
            self:RecreateActor()
        end
    end,

    OnScreenTransition = function (self, event)
        if not self.screen or self.screen ~= event.newScreen then
			return
		end

        self:RecreateBackground()
        self:RecreateActor()
    end,

    Edit = function (self, actorResource, prevScreen)
        if not (IsString(actorResource) or IsNil(actorResource)) then -- or IsTable(actorResource)) then
            error("a valid actor resource name was not specified")
        end
        ActorEditorScreen_actorResource = actorResource
        ActorEditorScreen_prevScreen = prevScreen
        self:RecreateActor()
    end,

    RecreateActor = function (self)
        self.theActor = nil -- Deletes the previous actor.  Not doing so will cause GuruEngine to work with the old actor, which may have unwanted attributes (color, etc.)
        if not ActorEditorScreen_actorResource then
            self.theActor = CreateInstance("Actor")
        elseif IsString(ActorEditorScreen_actorResource) then
            if Resources[ActorEditorScreen_actorResource] then
                self.theActor = Resources[ActorEditorScreen_actorResource]
            else
                error(string.format("ActorEditorScreen couldn't find a resource called, \"%s\"", ActorEditorScreen_actorResource))
            end
        --elseif IsTable(ActorEditorScreen_actorResource) then  -- this sort-of works, however a reload of lua resources may introduce a new table, especially if it refs into Resources[]
        --    self.theActor = ActorEditorScreen_actorResource
        else
            error(string.format("ActorEditorScreen couldn't load its actor as the Lua-global, 'ActorEditorScreen_actorResource' was of an unsupported Lua type, %s", type(ActorEditorScreen_actorResource)))
        end
    end,

    RecreateBackground = function (self)
        self.background = nil
        if ActorEditorScreen_background then
            if not RES_MGR:HasResourceInfo(ActorEditorScreen_background) then
                logprintf("Can't find background image for ActorEditorScreen.  ActorEditorScreen_background was \"%s\")\n", tostring(ActorEditorScreen_background))
            else
                self.background = {
                    class = "Actor",
                    image = ActorEditorScreen_background,
                    zorder = -1,
                }
            end
        end
    end,
}

