--[[
	Class: BuilderScreen
	Parent Class: Screen
	Properties: 
		All Actor Properties
		resourceName -- the name of the screen the builder screen is emulating.  The emulated screen should have a resource by this name stored in a file with this name ending in .lua.
--]]

Resources.BuilderScreen = Resources.Screen:new
{
	class = "BuilderScreen",
}


--
-- Creates a GURU_SCREENTRANSITION event handler that will call a BuilderScreen resources'
-- OnScreenShow() and OnScreenHide() methods if and when that particular screen is
-- switched-to or switched-away-from.
--
-- Example, using a BuilderScreen-made screen with a resource name, and a screen name of,
-- "MyScreen":
--
-- Resources.MyScreen = {
--     size = {800, 600},
--     color = "Black",
--     GURU_SCREENTRANSITION = CreateBuilderScreenTransitionHelper("MyScreen"),
--     firstArbitraryProperty = 12345,
--     secondArbitraryProperty = 67890,
--     OnScreenShow = function (self)
--         logprint(tostring(self.firstArbitraryProperty))
--     end,
--     OnScreenHide = function (self)
--         logprint(tostring(self.secondArbitraryProperty))
--     end,
-- }
--
-- If and wehn the screen created via the above example is switched to, "12345" will
-- get printed to the log.  When the screen is switched away from, "67890" will get
-- printed to the log.
--

function CreateBuilderScreenTransitionHelper(nameOfBuilderScreen)
	return function (eventData)
		local oldScreen = eventData.oldScreen
		local newScreen = eventData.newScreen

		if oldScreen and oldScreen.name == nameOfBuilderScreen then
			oldScreen.RootActor:OnScreenHide(newScreen)
		end

		if newScreen and newScreen.name == nameOfBuilderScreen then
			newScreen.RootActor:OnScreenShow(oldScreen)
		end
		
		return true
	end
end


--[[
	Scripts.Debug_RefreshDialog -- Helps launch a dialog via an overloaded BuilderScreen::Refresh call.

	To use, add this script to a BuilderScreen, passing in the name of the dialog
	as a parameter.  For example:

	(... from C++ code, inside and at the end of an overload of BuilderScreen::Refresh()):

	Script * refreshDialog = Script::Make("Debug_RefreshDialog");
	if (refreshDialog) {
		refreshDialog->PushParam<Actor *>(this);
		refreshDialog->PushParam<string>("NameOfDialog");

		//// OPTIONAL: Create the dialog via custom code.
		//refreshDialog->PushParam<std::function<int (LuaState *)> >([this] (LuaState * env) -> int {
		//    // create + start a dialog here, using custom C++ code
		//	return 0;
		//});

		RunScript(refreshDialog);
	}

--]]

local function Debug_RefreshDialog_DefaultStartDialog(screen, dialogName, pauseGame)
	local dialog = screen:StartDialog(dialogName, dialogName, pauseGame)

	-- Start dialog scripts:
	local preshowName = "Preshow_" .. dialogName
	if Scripts[preshowName] then
		RunScriptWithParams(dialog, preshowName)
	end

	local presentName = "Present_" .. dialogName
	if Scripts[preshowName] then
		AddScriptWithParams(dialog, presentName)
	end
end

Scripts.Debug_RefreshDialog = function (target, dialogName, startDialog)
	logprint(tostring(Script.current.id)..", Debug_RefreshDialog: "..tostring(dialogName))
	if (not IsString(dialogName)) or (dialogName == "") then
		error("1st argument (dialogName) must be a non-empty string")
	end

	if (not IsNil(startDialog)) and (not IsFunction(startDialog)) then
		error(string.format('3rd argument (startDialog) must either be a function, or nil, but was "%s"', type(startDialog)))
	end
	startDialog = startDialog or Debug_RefreshDialog_DefaultStartDialog

	-- Setup useful vars:
	local screen = target.screen
	--screen:LogAllScripts()

	-- Kill any pending "Debug_RefreshDialog" scripts
	local scripts = screen:GetTaggedScripts("Debug_RefreshDialog")
	for i,v in ipairs(scripts) do
		if v ~= Script.current then
			Script.current:Kill()
		end
	end

	-- Kill the screen's previous 'Present_' script to finish:
	local presentScript = "Present_"..screen.name
	if screen:HasTaggedScript(presentScript) then
		--logprint(tostring(Script.current.id)..", killing old Present_<screenName> scripts")
		screen:KillTaggedScripts(presentScript)
	end
	--screen:LogAllScripts()

	--
    -- HACK: Prevent this script from running too soon after its last
    --   run.  This is useful on OSX, whereby the engine's file watcher
    --   tends to run multiple times after a file gets saved.
	--
	-- To note, '_G' is a special variable used to access Lua globals.  It's
	-- used by this script to help access globals given their name, as a string.
	--
	--[[	-- DavidL: I disabled this on Jun 10, 2015, as it was preventing dialogs from coming up, in some cases (in Cascade's MenuScreen)
	local propertyNameForLastTickCount = string.format("lastScript_%s", tostring(dialogName))
	local lastTickCount = _G[propertyNameForLastTickCount]
	local deltaTicks = (APP.tickCount - (lastTickCount or 0))
	--logprintf("*****, %s,%s: %d - %d = %d\n", Script.current.name, tostring(Script.current.id), APP.tickCount, (lastTickCount or 0), deltaTicks)
	if not IsNil(lastTickCount) and (deltaTicks <= 250) and (APP.tickCount > 1) then
		if screen:IsDialogDisplaying(dialogName) then
			return
		end
	end
	_G[propertyNameForLastTickCount] = APP.tickCount
	--]]

    -- Stop the dialog:
	if screen:IsDialogDisplaying(dialogName) then
		--logprint("stopping dialog")
		screen:StopDialog(dialogName)

        -- Wait for the dialog to finish stopping:
    	while screen:IsDialogDisplaying(dialogName) do
    		--logprint("waiting...")
    		ScriptBreak()
    	end
    	--logprint("done waiting!")
    end

    -- Start a new dialog:
	--logprint(tostring(Script.current.id)..", creating dialog")
    local pauseGame = false
	local dialog = startDialog(screen, dialogName, pauseGame)

--[[
	-- DavidL: HACK code to get a dialog to ignore game pauses (so its scripts will run)
	if not dialog then
		dialog = screen:GetTopDialog(dialogName)
	end
	logprint(tostring(Script.current.id)..", created dialog: "..tostring(dialog))
	if dialog then
		--logprint(tostring(Script.current.id)..", setting ignore game pause")
		dialog.ignoreGamePause = true
	end
	if IsGamePaused() then
		ResumeGame()
	end
--]]
end
