--[[
	PointF_New
	
	Converts a pair of numbers into a value accessible via "x" and "y" properties.
		
	Returns:
		A value accessible with "x" and "y" fields, each of which evaluate to numbers.
	
	Examples:
		-- Assign {x=100, y=200} to foo.
		foo = PointF_New(100, 200)
		
		-- Assign {x=20, y=400} to foo.
		foo = PointF_New({20, 400})
		
		-- Assign {x=0.25, y=1.0} to foo.
		foo = PointF_New({x=0.25, y=1.0})
		
	Notes:
		This function is meant to convert a point into the same format that the C++ method,
		Point<float>::ToLuaObject would output.  It is also meant to take input in the same format
		that Point<float>::SetPoint would accept.  At some point in the future, it will likely use
		these methods, or whatever their counterparts are at the time, just so long as there is one
		method to convert a Point<float> to/from a Lua value.

		In the future, PointF_New may not return a table, but rather a userdata, which would point
		to an instance of Point<float>.  In this case, there will still be "x" and "y" fields,
		however it will not likely be possible to iterate (through the return value of PointF_New.)
--]]
function PointF_New(a, b)
	local x, y

	if type(a) == "table" then
		x = a.x or a[1]
		y = a.y or a[2]
	elseif type(a) == "number" and type(b) == "number" then
		x = a
		y = b
	else
		logprint(string.format("WARNING: PointF_New cannot process parameters of types, \"%s\" and \"%s\".",
			type(a),
			type(b)))
		return
	end
	
	return {x=x, y=y}
end

function Point_Magnitude(point)
	return math.sqrt((point.x*point.x) + (point.y*point.y))
end

function Point_Normalize(point, desiredMagnitude)
	local x = point.x;
	local y = point.y;
	
	local magnitude = Point_Magnitude(point)
	if (magnitude <= 0) then
		return point;
	else
		x = x / magnitude;
		y = y / magnitude;

		x = x * desiredMagnitude;
		y = y * desiredMagnitude;
		local newPoint = {x=x, y=y};
		return newPoint;
	end
end

Point = 
{
	type = "Point",
	
	fromTable = function(self, p1)
		--logprint("Point:fromTable() entered")
		local obj = {}
		obj.x = p1.x or p1[1]
		obj.y = p1.y or p1[2]
		--logprint("Point:fromTable() exited")
		return obj
	end,
	
	fromParams = function(self, p1, p2)
		--logprint("Point:fromParams() entered")
		local obj = {}
		obj.x = p1
		obj.y = p2
		--logprint("Point:fromParams() exited")
		return obj
	end,
	
	new = function(self, p1, p2)
		local p1 = p1 or 0
		local p2 = p2 or 0
		local obj = {}
		
		if type(p1) == "table" then
			obj = self:fromTable(p1)
		elseif ((type(p1) == "number" or type(p1) == "integer") and 
			    (type(p2) == "number" or type(p2) == "integer"))then
			obj = self:fromParams(p1, p2)
		end

		setmetatable(obj, self)
		self.__index = self
		return obj
	end,
	
	__tostring = function(self)
		--logprint(LOG_NORMAL, "Point:__tostring()")
		return ("{ x = ".. self.x .. ", y = ".. self.y.. " }")
	end
}

Size = 
{
	type = "Size",
	
	fromTable = function(self, p1)
		--logprint("Size:fromTable() entered")
		local obj = {}
		obj.x = p1.x or p1.width or p1[1]
		obj.y = p1.y or p1.height or p1[2]
		--logprint("Size:fromTable() exited")
		return obj
	end,

	fromParams = function(self, p1, p2)
		--logprint("Size:fromParams() entered")
		local obj = {}
		obj.x = p1
		obj.y = p2
		--logprint("Size:fromParams() exited")
		return obj
	end,
	
	new = function(self, p1, p2)
		local p1 = p1 or 0
		local p2 = p2 or 0
		local obj = {}
		
		if type(p1) == "table" then
			obj = self:fromTable(p1)
		elseif ((type(p1) == "number" or type(p1) == "integer") and 
			    (type(p2) == "number" or type(p2) == "integer"))then
			obj = self:fromParams(p1, p2)
		end

		obj.width = function() return obj.x end
		obj.height = function() return obj.y end

		setmetatable(obj, self)
		self.__index = self
		return obj
	end,

	width = function() 
		return obj.x or 0
	end,
	
	height = function() 
		return obj.y or 0 
	end,
	
	__tostring = function(self)
		--logprint(LOG_NORMAL, "Size:__tostring() entered")
		local width = self:width()
		local height = self:height()
		return ("{ width = ".. tostring(width) .. ", height = ".. tostring(height) .. " }")
	end,
}

function IsPoint(v)
	if not IsTable(v) then
		return false
	end
	
	if (#v == 2 and IsNumber(v[1]) and IsNumber(v[2])) or
	   (IsNumber(v.x) and IsNumber(v.y))
	then
		-- Make sure there are no other key+value pairs in the table.
		local pairCount = 0
		for k,v in pairs(v) do
			pairCount = pairCount + 1
			if pairCount > 2 then
				return false
			end
		end
		return (pairCount == 2)
	else
		return false
	end
end

