
--
-- Math helpers for Scripts.Interpolate
--

local SingleValueOps = {
	add = function (a,b) return a + b end,
	sub = function (a,b) return a - b end,
	mul = function (a,b) return a * b end,
	div = function (a,b) return a / b end
}

local MakeMultiValueOp = function (functor)
	return function (a,b,c)
		if IsNumber(b) then
			for k,v in pairs(a) do
				c[k] = functor(a[k],b)
			end
		else
			for k,v in pairs(a) do
				c[k] = functor(a[k],b[k])
			end
		end
		return c
	end
end

local MultiValueOps = {
	add = MakeMultiValueOp(SingleValueOps.add),
	sub = MakeMultiValueOp(SingleValueOps.sub),
	mul = MakeMultiValueOp(SingleValueOps.mul),
	div = MakeMultiValueOp(SingleValueOps.div),
}

--
-- Scripts.Interpolate
--
-- Linearly interpolates an object's property from its existing value to
-- something else over a given duration.
--
-- Values may either be a single number, or a table of values.  If a table
-- of values is provided, each value in that table will be updated, up to
-- one level in depth.
--
-- Scripts.Interpolate will always wait for the interpolation to finish
-- before returning to the caller.  If an asynchronous interpolation is
-- desired, then attaching a new script will be required.
--
function Scripts.Interpolate(paramA, paramB, dest, durationMS)
	-- Declare some local functions for getting and setting values.
	local valueGetter
	local valueSetter

	-- See if 'Interpolate' is working with an object's property.  If so, then
	-- set up a default value getter and setter.
	if IsUserdata(paramA) and IsString(paramB) then
		local targetObject = paramA
		local propertyName = paramB
		valueGetter = function () return targetObject[propertyName] end
		valueSetter = function (value) targetObject[propertyName] = value end
	elseif IsFunction(paramA) and IsFunction(paramB) then
		-- The value getter + setter were passed into 'Interpolate' as
		-- functions.  Use those directly.
		valueGetter = paramA
		valueSetter = paramB
	else
		-- Unable to define the value getter and setter.  Error-out.
		error("An invalid set of params were passed into Interpolate(): it was unable to figure out how to get and/or set the value in question.")
	end
	
	-- If the source and dest values are both points,
	-- then make sure the dest is in the same format as src (of either
	-- {x=?,y=?} or {?,?}).
	local src = valueGetter()
	if IsPoint(src) and IsPoint(dest) then
		if IsNumber(src.x) and IsNumber(src.y) and IsNumber(dest[1]) and IsNumber(dest[2]) then
			dest = PointF_New(dest)
		elseif IsNumber(src[1]) and IsNumber(src[2]) and IsNumber(dest.x) and IsNumber(dest.y) then
			dest = {dest.x, dest.y}
		end
	end
	
	-- Choose how to perform mathmatical operations, given the value of 'dest'.
	-- If this cannot be determined, then exit with an error.
	--
	-- It will be assumed here that 'dest' can be safely assigned to
	-- targetActor[propertyName].  If not, the interpolator will fail later
	-- on, when an attempt is made to do the assignment.
	local ops
	if IsNumber(dest) then
		ops = SingleValueOps
	elseif IsTable(dest) then
		ops = MultiValueOps
	else
		error(string.format("'dest' is an unhandle-able value of Lua type, '%s'.", type(dest)))
	end

	-- Declare, and possibily initialize, temporary storage.
	local valueRangeSize, timeToValuesMultiplier, tempStorageA, tempStorageB
	if IsTable(dest) then
		-- For multi-value interpolations, a set of tables will be
		-- created.  These will be used throughout the lifetime of the
		-- interpolation.
		valueRangeSize = {}
		timeToValuesMultiplier = {}
		tempStorageA = {}
		tempStorageB = {}
	end
	
	-- Calculate the range in which the value(s) has to travel.
	local src = valueGetter()
	valueRangeSize = ops.sub(dest, src, valueRangeSize) --dest - src

	-- Calculate the range in which time will travel.
	local startMS = APP:GetElapsedGameTime()
	local endMS = startMS + durationMS
	local currentMS = startMS
	local elapsedMS = 0
	
	-- time to values init
	timeToValuesMultiplier = ops.div(valueRangeSize, durationMS, timeToValuesMultiplier) --valueRangeSize / durationMS

	--print("INIT: " .. serialize(valueGetter()) .. " --> " .. serialize(dest) .. "  over  " .. durationMS)
	--print("...", "valueRangeSize = ", serialize(valueRangeSize), "timeToValuesMultiplier = ", serialize(timeToValuesMultiplier))
	
	-- Place the hosting coroutine/Script into an infinite loop, one that is
	-- expected to be woken up every now and then.  When the loop is awoken,
	-- the value will be updated, and a check will be made to see if enough
	-- time has elapsed.
	while true do
		--print("\tloop,", "currentMS =", currentMS)
		
		-- See if enough time has elapsed.  This will be done up-front,
		-- before the coroutine has been yielded (via ScriptBreak()), in
		-- case the first tick (of Scripts.Interpolate) has a zero duration.
		if endMS <= currentMS then
			-- Make sure the destination value gets set, then exit.
			valueSetter(dest)
			--logprint("DONE!")
			return
		end
		
		-- Yield the hosting coroutine/Script.
		ScriptBreak()
		
		-- Advance time, and the property-to-update.
		currentMS = APP:GetElapsedGameTime()
		elapsedMS = currentMS - startMS
		valueSetter(ops.add(src, ops.mul(timeToValuesMultiplier, elapsedMS, tempStorageA), tempStorageB))
	end
end


--
-- Scripts.QuadraticInterpolate
--
-- Interpolates an object's property from its existing value to something else 
-- over a given duration, accelerating from and decelerating to 0 velocity at the ends.
--
-- Values may either be a single number, or a table of values.  If a table
-- of values is provided, each value in that table will be updated, up to
-- one level in depth.
--
-- Scripts.Interpolate will always wait for the interpolation to finish
-- before returning to the caller.  If an asynchronous interpolation is
-- desired, then attaching a new script will be required.
--
function Scripts.QuadraticInterpolate(paramA, paramB, dest, durationMS)
	-- Declare some local functions for getting and setting values.
	local valueGetter
	local valueSetter

	-- See if 'Interpolate' is working with an object's property.  If so, then
	-- set up a default value getter and setter.
	if IsUserdata(paramA) and IsString(paramB) then
		local targetObject = paramA
		local propertyName = paramB
		valueGetter = function () return targetObject[propertyName] end
		valueSetter = function (value) targetObject[propertyName] = value end
	elseif IsFunction(paramA) and IsFunction(paramB) then
		-- The value getter + setter were passed into 'Interpolate' as
		-- functions.  Use those directly.
		valueGetter = paramA
		valueSetter = paramB
	else
		-- Unable to define the value getter and setter.  Error-out.
		error("An invalid set of params were passed into Interpolate(): it was unable to figure out how to get and/or set the value in question.")
	end
	
	-- If the source and dest values are both points,
	-- then make sure the dest is in the same format as src (of either
	-- {x=?,y=?} or {?,?}).
	local src = valueGetter()
	if IsPoint(src) and IsPoint(dest) then
		if IsNumber(src.x) and IsNumber(src.y) and IsNumber(dest[1]) and IsNumber(dest[2]) then
			dest = PointF_New(dest)
		elseif IsNumber(src[1]) and IsNumber(src[2]) and IsNumber(dest.x) and IsNumber(dest.y) then
			dest = {dest.x, dest.y}
		end
	end
	
	-- Choose how to perform mathmatical operations, given the value of 'dest'.
	-- If this cannot be determined, then exit with an error.
	--
	-- It will be assumed here that 'dest' can be safely assigned to
	-- targetActor[propertyName].  If not, the interpolator will fail later
	-- on, when an attempt is made to do the assignment.
	local ops
	if IsNumber(dest) then
		ops = SingleValueOps
	elseif IsTable(dest) then
		ops = MultiValueOps
	else
		error(string.format("'dest' is an unhandle-able value of Lua type, '%s'.", type(dest)))
	end

	-- Declare, and possibily initialize, temporary storage.
	local valueRangeSize, averageVelocity, acceleration, tempStorageA, tempStorageB
	if IsTable(dest) then
		-- For multi-value interpolations, a set of tables will be
		-- created.  These will be used throughout the lifetime of the
		-- interpolation.
		valueRangeSize = {}
		averageVelocity = {}
		halfwayPoint = {}
		acceleration = {}
		accelerationComponent = {}
		velocityComponent = {}
		finalTotal = {}
	end
	
	-- Calculate the range in which the value(s) has to travel.
	local src = valueGetter()
	valueRangeSize = ops.sub(dest, src, valueRangeSize) --dest - src
	
	halfwayPoint = ops.add(src,dest,halfwayPoint) --dest + source / 2
	halfwayPoint = ops.div(halfwayPoint,2,halfwayPoint)

	-- Calculate the range in which time will travel.
	local startMS = APP:GetElapsedGameTime()
	local endMS = startMS + durationMS
	local currentMS = startMS
	local elapsedMS = 0
	local halfwayMS = 0
	
	-- average velocity init
	averageVelocity = ops.div(valueRangeSize, durationMS, averageVelocity)

	-- We must reach double average velocity at peak. We must do so in half of duration MS. So the acceleration is averageVelocity*2/durationMS*0.5. Since duration MS is scalar, we'll just multiply it by .25 instead.
	acceleration = ops.div(averageVelocity, durationMS * 0.25, acceleration)

	--print("INIT: " .. serialize(valueGetter()) .. " --> " .. serialize(dest) .. "  over  " .. durationMS)
	--print("...", "valueRangeSize = ", serialize(valueRangeSize), "averageVelocity = ", serialize(averageVelocity))
	
	-- Place the hosting coroutine/Script into an infinite loop, one that is
	-- expected to be woken up every now and then.  When the loop is awoken,
	-- the value will be updated, and a check will be made to see if enough
	-- time has elapsed.
	while true do
		--print("\tloop,", "currentMS =", currentMS)
		
		-- See if enough time has elapsed.  This will be done up-front,
		-- before the coroutine has been yielded (via ScriptBreak()), in
		-- case the first tick (of Scripts.Interpolate) has a zero duration.
		if endMS <= currentMS then
			-- Make sure the destination value gets set, then exit.
			valueSetter(dest)
			--logprint("DONE!")
			return
		end
		
		-- Yield the hosting coroutine/Script.
		ScriptBreak()
		
		-- Advance time, and the property-to-update.
		currentMS = APP:GetElapsedGameTime()
		elapsedMS = currentMS - startMS
		if(elapsedMS < durationMS / 2) then
			--Position = 0.5a*t^2 + v(0)*t + p(0).
			--In acceleration phase, v(0)*t is 0, and p(0) is src.
			accelerationComponent = ops.mul(acceleration,elapsedMS * elapsedMS * 0.5, accelerationComponent)
			finalTotal = ops.add(src,accelerationComponent,finalTotal)
			valueSetter(finalTotal)
		else
			halfwayMS = elapsedMS - durationMS / 2
			--In deceleration phase, T is T minus half the time.
			--acceleration is now negative, v(0) is now twice average speed, 
			accelerationComponent = ops.mul(acceleration,halfwayMS * halfwayMS * -0.5, accelerationComponent)

			velocityComponent = ops.mul(averageVelocity,halfwayMS * 2, velocityComponent)

			--and p(0) is halfway between points
			
			finalTotal = ops.add(accelerationComponent, velocityComponent, finalTotal)
			finalTotal = ops.add(finalTotal, halfwayPoint, finalTotal)
			valueSetter(finalTotal)
		end
	end
end

--
-- Scripts.InterpolateGlobalPoint
-- 
-- Linearly interpolates an actor's alignment point, such as its center or its
-- bottom-right corner, to a specific point in the global coordinate space, over
-- a specific period of time.
--
function Scripts.InterpolateGlobalPoint(targetObject, rectAlignment, dest, durationMS)
	Scripts.Interpolate(
		function () return targetObject:GetGlobalPoint(rectAlignment) end,
		function (value) targetObject:SetGlobalPoint(rectAlignment, value) end,
		dest,
		durationMS)
end

--
-- Scripts.QuadraticInterpolateGlobalPoint
-- 
-- Quadratically interpolates an actor's alignment point, such as its center or its
-- bottom-right corner, to a specific point in the global coordinate space, over
-- a specific period of time.
--
function Scripts.QuadraticInterpolateGlobalPoint(targetObject, rectAlignment, dest, durationMS)
	Scripts.QuadraticInterpolate(
		function () return targetObject:GetGlobalPoint(rectAlignment) end,
		function (value) targetObject:SetGlobalPoint(rectAlignment, value) end,
		dest,
		durationMS)
end

