
--[[
	StartScreenEmitter
	
	Starts a particle emitter on the current screen.  The screen must have an instance of
	ParticleManager added to it as a child, which must be named, "particleManager".
	
	Params:
		params: A table of parameters.  See the "Named Params" section for more info.
		
	Named Params:
		alignmentTarget: An instance of Actor which, if specified, will cause the emitter to be
			positioned relative to the actor's global rect.
		alignmentPoint: Specifies where in the alignmentTarget that the emitter should be
			positioned.  Defaults to "Center".
		offset: Offsets the position of the emitter by a given amount.
		position: Specifies where to place the emitter.  This value will be ignored if
			alignmentTarget is set.
		resourceName: The name of the resource that describes the emitter.  If nil,
			"DefaultIdenticalParticleEmitter" will be used.
	
	Returns:
		nil
		
	Examples:
		-- Starts the "MyEmitter" emitter at position {100, 100} on the current screen.
		StartScreenEmitter
		{
			resourceName = "MyEmitter",
			position = {100, 100},
		}
		
		-- Starts the default emitter in the center of the current screen.
		StartScreenEmitter {}
		
		-- Starts the default emitter at position {200, 300} on the current screen.
		StartScreenEmitter { position = {200, 300} }
		
		-- Starts the "MyEmitter" emitter in the center of the current screen.
		StartScreenEmitter { resourceName = "MyEmitter" }
		
		-- Starts the "MyEmitter" emitter in the center of the actor, myActor.
		StartScreenEmitter
		{
			resourceName = "MyEmitter",
			alignmentTarget = myActor,
		}
		
		-- Starts the "MyEmitter" emitter 20 pixels down and to the right of the center of the
		-- actor, myActor.
		StartScreenEmitter
		{
			resourceName = "MyEmitter",
			alignmentTarget = myActor,
			offset = {20, 20},
		}
		
		-- Starts the "MyEmitter" emitter 20 pixels down and to the right of the top-left of the
		-- actor, myActor.
		StartScreenEmitter
		{
			resourceName = "MyEmitter",
			alignmentTarget = myActor,
			alignmentPoint = "TopLeft",
			offset = {20, 20},
		}
--]]

function StartScreenEmitter (params)
	local screen = CurrentScreen()

	-- Extract params, using a reasonable set of defaults.
	params = params or {}
	local resourceName = params.resourceName or "DefaultIdenticalParticleEmitter"
	
	-- Find a point to align the emitter to.
	local position = GetGlobalAlignmentPoint(params)
	
	-- Find a container (a ParticleManager)	to place particles in.
	local particleManager = screen:FindChild("particleManager", true)
	assert(particleManager, "`screen` does not have a particle manager")
	
	-- Create an emitter and start it.
	local emitter = InstantiateIdenticalParticleEmitter(resourceName, particleManager)
	particleManager:AddChild(emitter)
	emitter:MoveCenter(position)
	emitter:Start()
end


--[[
	Particle: Inherits from Actor
		All Actor Properties plus:
			alpha
			alphaStep
			lifetime
			position
			rotationStep
			velocity
			acceleration
			destAlpha
--]]
Resources.Particle = Resources.Actor:new
{
	class			= "Particle",
	alpha			= 1.0,
	alphaStep		= 0.0,
	lifetime		= 1000,
	position		= {0.0, 0.0},
	rotationStep	= 0.0,
	velocity		= {0.0, 0.0},
	acceleration	= {0.0, 0.0},
	destAlpha		= 1.0,
}


--[[
	ParticleManager: Inherits from Actor
--]]
Resources.ParticleManager = Resources.Actor:new
{
	class			= "ParticleManager",
}


--[[
	ParticleTemplate: Inherits from Object
		All Object Properties plus:
			lifetime
			alpha
			destAlpha
			size
			velocity
			acceleration
			color
			image
			randomizedVelocity
			randomizedPosition
			randomizedImage
			position
--]]
Resources.ParticleTemplate = Resources.Object:new
{
	class				= "ParticleTemplate",
	lifetime			= 1000,
	alpha				= 1.0,
	destAlpha			= 1.0,
	size				= {0, 0},
	velocity			= {0.0, 0.0},
	acceleration		= {0.0, 0.0},
	color				= "White",
	image				= "",
	randomizedVelocity	= false,
	randomizedPosition	= false,
	randomizedImage		= {},
	position			= {0.0, 0.0},
	
}

--[[
	ParticleEmitter: Inherits from Actor
		All Actor Properties plus:
			emitCount
			interval
			destroyWhenStopped
			delay
--]]
Resources.ParticleEmitter = Resources.Actor:new
{
	class				= "ParticleEmitter",
	emitCount			= 1000,
	interval			= 10,
	destroyWhenStopped	= false,
	delay				= 0,
}

--[[
	Class: IdenticalParticleEmitter
	Parent Class: Actor
	Description: 
	Properties:
		All Actor Properties
		particleTemplate
--]]
Resources.IdenticalParticleEmitter = Resources.ParticleEmitter:new
{
	class				= "IdenticalParticleEmitter",
	particleTemplate 	= {}	
}



Resources.DefaultIdenticalParticleEmitter = Resources.IdenticalParticleEmitter:new
{
	resource = "IdenticalParticleEmitter",

    particleTemplate =
    {
		acceleration = {0, 250},
        alpha = 255,
        destAlpha = 0,
        lifetime = 750,
        position = {0, 0},

        randomizedImage =
        {
			"sparkle_b.png",
			"sparkle_g.png",
			"sparkle_o.png",
			"sparkle_pi.png",
			"sparkle_r.png",
			"sparkle_y.png",
		},

        randomizedVelocity =
        {
			minRadius = 150,
			maxRadius = 250,
			minAngle = 0,
			maxAngle = 2 * math.pi,
        },
    },

    numParticles = 25,
    emitCount = 1,
    interval = 100,
}
