
--
-- Shorthand versions of resource functions:
--
function mount(...)
    if #{...} == 0 then     -- if arg-count is 0 ...
        return LogMounts()
    else
        return MountBundleAtResource(...)
    end
end

function unmount(...)
    return UnmountBundleAtResource(...)
end

function needsremount(...)
    return IsBundleAtResourceRemountPending(...)
end

function remount(...)
    local didUnmount, err = UnmountBundleAtResource(...)
    if not didUnmount then
        return didUnmount, err
    end
    return MountBundleAtResource(...)
end

function logres(...)
    return LogResourceInfo(...)
end

-- Iterates through a table of properties, making sure that keys equalling
-- "resource" and "resources" are iterated before all others.
--
-- To note, this function supports resources that list either a "resource" or
-- a "resources" tag, but not both.  In the future, it may support both,
-- however no cases in any current project (Trinklit Mystery and GuruEngine) or
-- recent projects (Trinklit Supreme) do this.

function propertyIter(t)
	return nextProperty, t, nil
end


-- Retrieves a list of all named resources given a resource tree.
local function buildFlattenedResourceList (resource, outputList)
    local i, k
    local ignoreResourceProperty = ignoreResourceProperty or false
    
    --[[	-- DavidL: Disabled this on 2010-04-19, as "defaults" does not appear to be getting used, and to allow ClassManager to be simplified a bit.
    if (resource.class) then
		local classObject = Classes[resource.class]
		assert(classObject)
		if classObject.defaults then
			table.insert(outputList, classObject.defaults)
		end
	end
	--]]

	if (resource.resource) then
		buildFlattenedResourceList(Resources[resource.resource], outputList)
	end
	
    if (resource.resources) then
		if type(resource.resources) == 'string' then
			buildFlattenedResourceList(Resources[resource.resources], outputList)
		else
			for i, k in pairs(resource.resources) do
				buildFlattenedResourceList(Resources[k], outputList)
			end
		end
    end

    table.insert(outputList, resource)
end

function GetFlattenedResourceList(resource)
	local list = {}
		if(resource.class and Resources[resource.class]) then
			resourceList = {}
			table.insert(resourceList, Resources[resource.class])
		end
		
		if (resource.resources) then
			if type(resource.resources) == 'string' then
				buildFlattenedResourceList(Resources[resource.resources], outputList)
			else
				for i, k in pairs(resource.resources) do
					buildFlattenedResourceList(Resources[k], outputList)
				end
			end
		end
	return list	
end

-- Iterates through the key/value pairs in a list of tables.
local function orderedTableIter (baseTable)
    local i = next(baseTable)	-- an index into baseTable
    local k						-- an index into baseTable[i]
    return function ()
               k = next(baseTable[i], k)

               while not k do
                   i = next(baseTable, i)
                   if not i then
                       return
                   end
                   k = next(baseTable[i], k)
               end

               return k, baseTable[i][k]
           end
end
	

-- Iterates through all key/value pairs in a resource tree, visiting all parent
-- resources first.
function resourceContentIter (resource, resList)
	-- resList is a flattened list of resources that 'resource' inherits from
	-- it's use as a parameter is optional, and is rarely needed
	
    if not resList then
		resList = {}
	    buildFlattenedResourceList(resource, resList)
	end

	return orderedTableIter(resList)
end

-- Finds all values with a specific key in a tree of resources.
function findKeysInResourceTree (keyName, resourceTree)
	local output = {}

	for k, v in resourceContentIter(resourceTree) do
		if k == keyName then
			table.insert(output, v)
		end
	end
	
	return output
end




--
-- Experimental resource flattening code.
--
-- This code is used to see if it is possible to flatten a resource and its
-- sub-resources into a single table before it gets applied to a table.
--
-- More info on this experimental feature is available in Object::ApplyProperties().
--

local tab_count = 0

local function get_tab_string()
	local str = " "
	for i = 1,tab_count do
		str = str .. "----"
	end
	return str
end

local function resflatten_dbg_print(...)
	-- DavidL note: uncomment this to have the resource flattening algorithm print
	-- out the contents of each resource as it gets flattened.
	--return print(...)
end

function DoFlattenResource(inputTable, outputTable)
	-- Error checking.
	if not IsTable(inputTable) then
		error("The 1st argument, 'inputTable', was not a table")
	end

	if not IsTable(outputTable) then
		error("The 2nd argument, 'outputTable', was not a table")
	end
	
	-- Debug-printing whatnot.
	tab_count = tab_count + 1
	
	-- Add entries from the 'resources' property, if available.
	if inputTable.resources then
		if IsTable(inputTable.resources) then
			resflatten_dbg_print(string.format("%s resources (as table)", get_tab_string()))
			for k,v in pairs(inputTable.resources) do
				if IsString(k) and IsTable(Resources[k]) then
					DoFlattenResource(Resources[k], outputTable)
				end
			end
		elseif IsString(inputTable.resources) and IsTable(Resources[inputTable.resources]) then
			-- Try treating 'resources' as a string (that represents the name of a resource.)
			-- DavidL: This pattern (specifying 'resources' as a string) has been used throughout
			-- various screens and in-game code.
			resflatten_dbg_print(string.format("%s resources (as string: %s)", get_tab_string(), inputTable.resources))
			DoFlattenResource(Resources[inputTable.resources], outputTable)
		end
	end

	-- Add entries from the 'resource' property, if available.
	if inputTable.resource then
		if IsTable(inputTable.resource) then
			-- Try treating 'resource' as a table (that contains a list of resource names.)
			-- DavidL: I don't think this pattern (specifying 'resource' as a table) has ever
			-- been used, however the addition of code to deal with such seems like a better
			-- alternative than not having it, given that 'resources' is treated as both a
			-- table and a string.
			resflatten_dbg_print(string.format("%s resource (as table)", get_tab_string()))
			for k,v in pairs(inputTable.resource) do
				if IsString(k) and IsTable(Resources[k]) then
					DoFlattenResource(Resources[k], outputTable)
				end
			end
		elseif IsString(inputTable.resource) and IsTable(Resources[inputTable.resource]) then
			resflatten_dbg_print(string.format("%s resource (as string: %s)", get_tab_string(), inputTable.resource))
			DoFlattenResource(Resources[inputTable.resource], outputTable)
		end
	end
	
	-- Copy all values from inputTable to outputTable, except 'resource' and 'resources',
	-- the contents of which should already have been dereferenced and added.
	for k,v in pairs(inputTable) do
		if k ~= "resource" or k ~= "resources" then
			-- DavidL: tostring(v) seems to occasionally return nil, which can cause string.format to crash.
			-- I'm not sure why it would ever return nil, rather than a stringified representation of something
			-- else.  In any case, I added some code to handle this case.  It makes the string, "<<< NIL >>>",
			-- be used when tostring(v) returns nil.
			resflatten_dbg_print(string.format("%s %s: %s", get_tab_string(), k, tostring(v) or "<<< NIL >>>"))

			outputTable[k] = v
		end
	end
	
	-- Debug-printing whatnot.
	tab_count = tab_count - 1
end

-- DavidL note: Uncomment this line to enable a simple cache for resource flattening.
--local flattenedResourceCache = {}

function FlattenResource(inputTable)
	resflatten_dbg_print("FlattenResource START:", tostring(inputTable))
	
	-- See if a cached result is available.  If so, return it, rather
	-- than recomputing the result.
	if flattenedResourceCache and flattenedResourceCache[inputTable] then
		resflatten_dbg_print("FlattenResource END, cached result returned:", tostring(inputTable))
		return flattenedResourceCache[inputTable]
	end
	
	-- Flatten the resource.
	local outputTable = {}
	DoFlattenResource(inputTable, outputTable)

	-- Cache the result (provided the cache is enabled.)
	if flattenedResourceCache then
		flattenedResourceCache[inputTable] = outputTable
	end

	-- Return the computed result.
	resflatten_dbg_print("FlattenResource END:", tostring(inputTable))
	return outputTable
end


