-- Load a file if it exists in the directory hierarchy.
function LoadOptionally(fileName)
    if not Resources[fileName] then
        return
    end

    Load(fileName)
end

function LoadAll(...)
    local compiled = {}
    for _, t in ipairs({...}) do
        for _, resFile in ipairs(t) do
            --logprintf("COMPILING: %s\n", SerializeToString(resFile))

            local resFileName, optional
            if type(resFile) == "string" then
                resFileName = resFile
                optional = false
            else
                resFileName = resFile[1]
                optional = resFile.optional
            end

            if optional then
                if not RES_MGR:HasResourceInfo(resFileName) then
                    continue
                end
            end

            ---logprintf("COMPILING: %s\n", tostring(resFileName))
            local result = LuaResourceCompile(resFileName)
            if type(result) ~= 'function' then
                local err = tostring(result)

                local fullErr = {"\n-------------------------------------------------------\n"}
                table.insert(fullErr, string.format("Unable to compile the Lua file: %s\n", tostring(resFileName)))

                -- Try printing out the location of the error, in a format that Visual Studio likes:
                local lineNumberStr = err:match("\"]:(%d+)")
                local fullPathToFile
                if RES_MGR then
                    fullPathToFile = FSPathCanonicalize(RES_MGR:GetPathOfResource(resFileName))
                    fullPathToFile = fullPathToFile:gsub("^(%a):/", "%1:\\")
                end
                if type(fullPathToFile) == "string" and type(lineNumberStr) == "string" then
                    table.insert(fullErr, string.format("\n%s(%s)\n", fullPathToFile, lineNumberStr))
                end

                table.insert(fullErr, string.format("\n%s\n", tostring(err)))
                table.insert(fullErr, "-------------------------------------------------------\n")
                local fullErrStr = table.concat(fullErr, "")
                logprintf("%s\n", fullErrStr)
                return fullErrStr
            end
            table.insert(compiled, {
                resFileName = resFileName,
                run = result
            })
        end
    end
    for _, c in ipairs(compiled) do
        --logprintf("RUNNING: %s\n", tostring(c.resFileName))
        c.run()
    end
    return true
end

--SetLogLevel(3)

EngineScriptFiles = {
    {"Script.lua"},
    {"Utility.lua"},
    {"Engine_Const.cfg"},
    {GetSystemResourceName("ConstConfigFile"),          optional = true},
    {"Engine_PlayerSpecificKeys.cfg"},
    {GetSystemResourceName("PlayerSpecificConfigKeys"), optional = true},
    {"site.cfg",                                        optional = true},

-- Load and initialize debugger
---[[ remove additional '-' at the beginning of this line to disable debugger
    {"Debug.lua",                                   }, -- reloadOk = false,
--]]

-- NOTE: The false parameter after each filename specifies that we don't
-- want to reloxad that file if it is already loaded

    {"Resource.lua",                                }, -- reloadOk = false,

    {"Localization.lua",                            }, -- reloadOk = false,
    {"EngineStrings.lua",                           }, -- reloadOk = true,
    {"AppStrings.lua",          optional = true,    }, -- reloadOk = true,
    {"AppURLs.lua",                                 }, -- reloadOk = true,
    {"AppStoreIDs.lua",                             }, -- reloadOk = true,
    {"Object.lua",                                  }, -- reloadOk = false,
    {"EventMap.lua",                                }, -- reloadOk = false,
    {"Enums.lua",                                   }, -- reloadOk = false,
    {"Geometry.lua",                                }, -- reloadOk = false,
    {"Colors.lua",                                  }, -- reloadOk = false,
    {"Keys.lua",                                    }, -- reloadOk = false,
    {"Font.lua",                                    }, -- reloadOk = false,
    {"Margins.lua",                                 }, -- reloadOk = false,
    {"Actor.lua",                                   }, -- reloadOk = false,
    {"Label.lua",                                   }, -- reloadOk = false,
    {"Draggable.lua",                               }, -- reloadOk = false,
    {"Border.lua",                                  }, -- reloadOk = false,
    {"Button.lua",                                  }, -- reloadOk = false,
    {"TextBox.lua",                                 }, -- reloadOk = false,
    {"Screen.lua",                                  }, -- reloadOk = false,
    {"PlayerPicture.lua",                           }, -- reloadOk = false,
    {"Templates.lua",                               }, -- reloadOk = false,
    {"ListBox.lua",                                 }, -- reloadOk = false,
    {"LinearFader.lua",                             }, -- reloadOk = false,
    {"Dialog.lua",                                  }, -- reloadOk = false,
    {"CommonDialogs.lua",                           }, -- reloadOk = false,
    {"TournamentDialogs.lua",                       }, -- reloadOk = false,
    {"Particle.lua",                                }, -- reloadOk = false,
    {"Player.lua",                                  }, -- reloadOk = false,
    {"NoteManager.lua",                             }, -- reloadOk = false,
    {"Tournament.lua",                              }, -- reloadOk = false,
    {"PlayerManager.lua",                           }, -- reloadOk = false,
    {"Rotator.lua",                                 }, -- reloadOk = false,
    {"Scaler.lua",                                  }, -- reloadOk = false,
    {"SplashScreen.lua",                            }, -- reloadOk = false,
    {"ScoreFlash.lua",                              }, -- reloadOk = false,
    {"BuilderScreen.lua",                           }, -- reloadOk = false,
    {"hgeparticle.lua",                             }, -- reloadOk = false,
    {"ParticleEditor.lua",                          }, -- reloadOk = false,
    {"ParticleEditorScreen.lua",                    }, -- reloadOk = false,
    {"ActivityIndicators.lua",                      }, -- reloadOk = false,
    {"GameCenterViews.lua",                         }, -- reloadOk = false,
    {"Interpolate.lua",                             }, -- reloadOk = false,
    {"Notifications.lua",                           }, -- reloadOk = false,
    {"TopScoreViewManager.lua",                     }, -- reloadOk = false,
    {"ScrollableArea.lua",                          }, -- reloadOk = false,
    {"Gifting.lua",                                 }, -- reloadOk = false,
    {"ActorEditorScreen.lua",                       }, -- reloadOk = false,

--These two may be created by the specific app as a way to override defaults, but are optional.
    {"PlayerDefaults.lua",      optional = true     },
    {"DummyHighScores.lua",     optional = true     },
}   -- END of EngineScriptFiles
