

--[[
	RotateAtSpeed

	Rotates an actor at a given speed.

	Params:
		actor: The actor to rotate.
		speed: The speed to rotate at, represented as an Angle.
		duration: (default = nil) The amount of time to rotate, measured in ms.  To rotate indefinitely,
			pass in 'nil'.
		waitForCompletion: (default = nil) Indicates whether or not the function should wait for
			the rotation to finish before returning.  See the documentation for IndicatesWaiting in
			Script.lua for a list of valid parameters (note: to wait, either pass in boolean true or
			"wait", otherwise, pass in nil, boolean false, or "nowait").
			
			If this function is called outside of a coroutine-based script (via GuruEngine's
			"Script" class), then this parameter must be set to false, or it will fail.
		
	Returns:
		nil

	Examples:
		-- Rotates myActor at one revolution/second, indefinitely.
		RotateAtSpeed(myActor, {revolutions=1.0})

		-- Rotates myActor at 720 degrees/second for three seconds.
		RotateAtSpeed(myActor, {degrees=720}}, 3000)
--]]

function RotateAtSpeed(actor, speed, duration, waitForCompletion)
	local rotator = InstantiateRotator()
	actor.rotator = rotator
	rotator:StartRotatingAtSpeed(speed, duration)
	
	if IndicatesWaiting(waitForCompletion) then
		WaitForRotationCompletion(actor)
	end
end


--[[
	RotateByAmount

	Rotates an actor by an amount.

	Params:
		actor: The actor to rotate.
		theta: The amount to rotate, represented as an Angle.
		duration: The amount of time to rotate, measured in ms.
		waitForCompletion: (default = nil) Indicates whether or not the function should wait for
			the rotation to finish before returning.  See the documentation for IndicatesWaiting in
			Script.lua for a list of valid parameters (note: to wait, either pass in boolean true or
			"wait", otherwise, pass in nil, boolean false, or "nowait").
			
			If this function is called outside of a coroutine-based script (via GuruEngine's
			"Script" class), then this parameter must be set to false, or it will fail.
		
	Returns:
		nil

	Examples:
		-- Rotates myActor one full revolution, clockwise, over the span of two seconds.
		RotateByAmount(myActor, {revolutions=1.0}, 2000)
		
		-- Rotates myActor one full revolution, counter-clockwise, over the span of two seconds.
		RotateByAmount(myActor, {revolutions=-1.0}, 2000)
--]]

function RotateByAmount(actor, theta, duration, waitForCompletion)
	local rotator = InstantiateRotator()
	actor.rotator = rotator
	rotator:StartRotatingByAmount(theta, duration)
	
	if IndicatesWaiting(waitForCompletion) then
		WaitForRotationCompletion(actor)
	end
end


--[[
	RotateTo
	
	Rotates an actor to a given angle from its current one over a specified period of time.
	
	Params:
		actor: The actor to rotate.
		endAngle: The angle to rotate the actor to.
		duration: The amount of time to rotate, measured in ms.
		waitForCompletion: (default = nil) Indicates whether or not the function should wait for
			the rotation to finish before returning.  See the documentation for IndicatesWaiting in
			Script.lua for a list of valid parameters (note: to wait, either pass in boolean true or
			"wait", otherwise, pass in nil, boolean false, or "nowait").
			
			If this function is called outside of a coroutine-based script (via GuruEngine's
			"Script" class), then this parameter must be set to false, or it will fail.
		
	Examples:
		-- Rotates myActor to a quarter revolution (90 degrees) over the span of two seconds.
		RotateTo(myActor, {revolutions=0.25}, 2000)
--]]

function RotateTo(actor, endAngle, duration, waitForCompletion)
	local currentRotation = actor.rotationAngle
	local theta = Angle_Subtract(endAngle, currentRotation)
	RotateByAmount(actor, theta, duration)
	
	if IndicatesWaiting(waitForCompletion) then
		WaitForRotationCompletion(actor)
	end
end


--[[
	RotateIn
	
	Saves the rotation of an actor, immediately sets a new rotation, makes the actor visible,
	and finally rotates the actor to its original rotation, optionally waiting for the rotation to
	finish.
	
	Params:
		actor: The actor to rotate.
		startAngle: The angle of the actor at the start of the timed rotation.
		duration: The amount of time to rotate, measured in ms.
		waitForCompletion: (default = nil) Indicates whether or not the function should wait for
			the rotation to finish before returning.  See the documentation for IndicatesWaiting in
			Script.lua for a list of valid parameters (note: to wait, either pass in boolean true or
			"wait", otherwise, pass in nil, boolean false, or "nowait").
			
			If this function is called outside of a coroutine-based script (via GuruEngine's
			"Script" class), then this parameter must be set to false, or it will fail.
			
	Returns:
		nil
		
	Examples:
		-- Rotates myActor in from an eighth of a revolution over the span of two seconds.
		RotateIn(myActor, {revolutions=0.125}, 2000)
--]]

function RotateIn(actor, startAngle, duration, waitForCompletion)
	actor.visible = true

	local originalRotation = actor.rotationAngle
	actor.rotationAngle = startAngle
	RotateTo(actor, originalRotation, duration)
	
	if IndicatesWaiting(waitForCompletion) then
		WaitForRotationCompletion(actor)
	end
end


--[[
	RotateOut
	
	Rotates an actor from its current rotation to a specified angle, optionally waiting for the
	rotation to complete, in which case the actor's original rotation will be restored and the
	actor made invisible.
	
	Params:
		actor: The actor to rotate.
		endAngle: The angle to rotate the actor to.
		duration: The amount of time to rotate, measured in ms.
		waitForCompletion: (default = nil) Indicates whether or not the function should wait for
			the rotation to finish before returning.  See the documentation for IndicatesWaiting in
			Script.lua for a list of valid parameters (note: to wait, either pass in boolean true or
			"wait", otherwise, pass in nil, boolean false, or "nowait").
			
			If this function is called outside of a coroutine-based script (via GuruEngine's
			"Script" class), then this parameter must be set to false, or it will fail.
			
			If the function is told to wait, then after the actor finishes rotating, its original
			rotation will be restored, in addition to the actor being made invisible.
			
	Returns:
		nil
--]]

function RotateOut(actor, endAngle, duration, waitForCompletion)
	local originalRotation = actor.rotationAngle
	RotateTo(actor, endAngle, duration)

	if IndicatesWaiting(waitForCompletion) then
		WaitForRotationCompletion(actor)

		actor.rotationAngle = originalRotation
		actor.visible = false
	end
end


--[[
	IsRotating
	
	Determines if an actor is actively rotating (over time.)
	
	Params:
		actor: The actor to query.
		
	Returns:
		true if the actor is actively rotating, false otherwise.
--]]

function IsRotating(actor)
	return actor.rotator and actor.rotator:IsRunning()
end


--[[
	WaitForRotationCompletion

	Waits for an actor to finish rotating.
	This method will fail unless it is run in a coroutine-based script.
	
	Params:
		actor: The actor to wait for.
		
	Returns:
		nil
--]]

function WaitForRotationCompletion(actor)
	while IsRotating(actor) do
		ScriptBreak()
	end
end

