

--[[
	ScaleAtVelocity

	Scales an actor at a given velocity.

	Params:
		actor: The actor to scale.
		velocity: The velocity to scale at, represented as a pair of numbers: X axis speed and Y axis speed
		duration: (default = nil) The amount of time to scale, measured in ms.  To scale indefinitely,
			pass in 'nil'.
		waitForCompletion: (default = nil) Indicates whether or not the function should wait for
			the scaling to finish before returning.  See the documentation for IndicatesWaiting in
			Script.lua for a list of valid parameters (note: `true` for waiting, `false` for not
			is fine.)
			
			If this function is called outside of a coroutine-based script (via GuruEngine's
			"Script" class), then this parameter must be set to false, or it will fail.
		
	Returns:
		nil

	Examples:
		-- Linearly increases an Actor's scale by 1.0 on both axes every second, indefinitely.
		ScaleAtVelocity(myActor, {1.0, 1.0})

		-- Linearly increases an Actor's scale by 0.5 on both axes for two seconds.
		ScaleAtVelocity(myActor, {0.5, 0.5}, 2000)

		-- Linearly increases an Actor's scale by 1.0 on the Y axis for one and a half seconds.
		ScaleAtVelocity(myActor, {0.0, 1.0}, 1500)
--]]

function ScaleAtVelocity(actor, velocity, duration, waitForCompletion)
	local scaler = InstantiateScaler()
	actor.scaler = scaler
	scaler:StartScalingAtVelocity(velocity, duration)
	
	if IndicatesWaiting(waitForCompletion) then
		ScriptDelay(duration,"DialogTimers")
	end
end


--[[
	ScaleByAmount

	Scales an actor by an amount.

	Params:
		actor: The actor to scale.
		amount: The amount to scale, represented as a pair of numbers: X axis scaling and Y axis scaling
		duration: The amount of time to scale, measured in ms.
		waitForCompletion: (default = nil) Indicates whether or not the function should wait for
			the scaling to finish before returning.  See the documentation for IndicatesWaiting in
			Script.lua for a list of valid parameters (note: `true` for waiting, `false` for not
			is fine.)
			
			If this function is called outside of a coroutine-based script (via GuruEngine's
			"Script" class), then this parameter must be set to false, or it will fail.
		
	Returns:
		nil

	Examples:
		-- Adds a total of 1.0 to each of myActor's scaling axes over the span of two seconds.
		ScaleByAmount(myActor, {1.0, 1.0}, 2000)
	
		-- Scales myActor to half of its current scale over the span of two seconds.
		local s1 = myActor.extraScale
		local s2 = {x = (s1.x / 2), y = (s1.y / 2)}
		local amount = {x = (s2.x - s1.x), y = (s2.y - s1.y)}
		ScaleByAmount(myActor, amount, 2000)
		
--]]

function ScaleByAmount(actor, amount, duration, waitForCompletion)
	local scaler = InstantiateScaler()
	actor.scaler = scaler
	scaler:StartScalingByAmount(amount, duration)
	
	if IndicatesWaiting(waitForCompletion) then
		ScriptDelay(duration,"DialogTimers")
	end
end


--[[
	ScaleTo
	
	Scales an actor from its current scale to a given one over a period of time.
	
	Params:
		actor: The actor to scale.
		endScale: The value to scale the actor to.
		duration: The amount of time to scale, measured in ms.
		waitForCompletion: (default = nil) Indicates whether or not the function should wait for
			the scaling to finish before returning.  See the documentation for IndicatesWaiting in
			Script.lua for a list of valid parameters (note: `true` for waiting, `false` for not
			is fine.)
			
			If this function is called outside of a coroutine-based script (via GuruEngine's
			"Script" class), then this parameter must be set to false, or it will fail.
		
	Examples:
		-- Scales myActor to nothing over the span of three seconds.
		ScaleTo(myActor, {0.0, 0.0}, 3000)

		-- Scales myActor to half of its current scale over the span of two seconds.
		local s1 = myActor.extraScale
		local s2 = {x = (s1.x / 2), y = (s1.y / 2)}
		ScaleTo(myActor, s2, 2000)
--]]

function ScaleTo(actor, endScale, duration, waitForCompletion)
	local currentScale = actor.extraScale
	endScale = PointF_New(endScale)
	local amount = {endScale.x - currentScale.x, endScale.y - currentScale.y}
	ScaleByAmount(actor, amount, duration)
	
	if IndicatesWaiting(waitForCompletion) then
		ScriptDelay(duration,"DialogTimers")
	end
end


--[[
	ScaleIn
	
	Saves the scale of an actor, immediately sets a new scale, makes the actor visible,
	and finally scales the actor to its original scale, optionally waiting for the scaling to
	finish.
	
	Params:
		actor: The actor to scale.
		startScale: The scale of the actor at the start of the timed scaling.
		duration: The amount of time to scale, measured in ms.
		waitForCompletion: (default = nil) Indicates whether or not the function should wait for
			the scaling to finish before returning.  See the documentation for IndicatesWaiting in
			Script.lua for a list of valid parameters (note: `true` for waiting, `false` for not
			is fine.)
			
			If this function is called outside of a coroutine-based script (via GuruEngine's
			"Script" class), then this parameter must be set to false, or it will fail.
			
	Returns:
		nil
		
	Examples:
		-- Scales myActor in from half its default size over the span of two seconds.
		ScaleIn(myActor, {0.5, 0.5}, 2000)
--]]

function ScaleIn(actor, startScale, duration, waitForCompletion)
	actor.visible = true
	
	local originalScale = actor.extraScale
	actor.extraScale = startScale
	ScaleTo(actor, originalScale, duration)
	
	if IndicatesWaiting(waitForCompletion) then
		ScriptDelay(duration,"DialogTimers")
	end
end


--[[
	ScaleOut
	
	Scales an actor from its current scale to a specified one, optionally waiting for the
	scaling to complete, in which case the actor's original scale will be restored and the
	actor will made invisible.
	
	Params:
		actor: The actor to scale.
		endScale: The amount to scale the actor to.
		duration: The amount of time to scale, measured in ms.
		waitForCompletion: (default = nil) Indicates whether or not the function should wait for
			the scaling to finish before returning.  See the documentation for IndicatesWaiting in
			Script.lua for a list of valid parameters (note: `true` for waiting, `false` for not
			is fine.)
			
			If this function is called outside of a coroutine-based script (via GuruEngine's
			"Script" class), then this parameter must be set to false, or it will fail.
			
			If the function is told to wait, then after the actor finishes scaling, its original
			scale will be restored, in addition to the actor being made invisible.
			
	Returns:
		nil
--]]

function ScaleOut(actor, endScale, duration, waitForCompletion)
	local originalScale = actor.extraScale
	ScaleTo(actor, endScale, duration)

	if IndicatesWaiting(waitForCompletion) then
		ScriptDelay(duration,"DialogTimers")

		actor.extraScale = originalScale
		actor.visible = false
	end
end


--[[
	PopIn
	
	Immediately scales an actor to nothing.  Then it sets its "visible" property to true, scales it
	in over time, usually to a size slightly larger than normal.  Finally it scales the actor to
	its normal size.
	
	This function will not return until all scaling is complete.  Furthermore, it cannot be called
	outside of a coroutine-based script.
	
	Params:
		actor: The actor to scale.
		adjustments: A table of values adjusting aspects of the animation.  These include:
			expandTo: The amount to initially scale to.
			expandTime: How long the initial expansion should take.
			shrinkTime: How long the final shrink (to normal) should take.
			
	Returns:
		nil
			
	Examples:
		-- Pops myActor in using default values.
		PopIn(actor)
		
		-- Pops myActor in, modifying the amount it should initially scale to.
		PopIn(actor, {expandTo={1.5, 0.25})
		
		-- Pops myActor in, modifying the amount of time it should take to expand and shrink the
		-- actor.
		PopIn(actor, {expandTime=500, shrinkTime=250})
--]]

function PopIn(actor, adjustments)
	local expandTo		= GetParam(adjustments, "expandTo",		{1.2, 1.2})
	local expandTime	= GetParam(adjustments, "expandTime",	250)
	local shrinkTime	= GetParam(adjustments, "shrinkTime",	100)
	
	actor.visible = true

	actor.extraScale = {0.0, 0.0}
	ScaleTo(actor, expandTo, expandTime)
	ScriptDelay(expandTime,"DialogTimers")
	
	ScaleTo(actor, {1.0, 1.0}, shrinkTime)
	ScriptDelay(shrinkTime,"DialogTimers")
end


--[[
	PopOut
	
	Scales an actor, over time, to a value slightly larger than normal (normal being {1.0, 1.0}).
	Then it scales it to nothing ({0.0, 0.0}) over an additional span of time.  Finally, it sets
	the actor's "visible" property to false and resets the scale to its original value.
	
	This function will not return until all scaling is complete.  Furthermore, it cannot be called
	outside of a coroutine-based script.
	
	Params:
		actor: The actor to scale.
		adjustments: A table of values adjusting aspects of the animation.  These include:
			expandTo: The amount to initially scale to.
			expandTime: How long the initial expansion should take.
			shrinkTime: How long the final shrink (to normal) should take.
			
	Returns:
		nil
			
	Examples:
		-- Pops myActor out using default values.
		PopOut(actor)
		
		-- Pops myActor out, modifying the amount it should initially scale to.
		PopOut(actor, {expandTo={1.5, 0.25})
		
		-- Pops myActor out, modifying the amount of time it should take to expand and shrink the
		-- actor.
		PopOut(actor, {expandTime=500, shrinkTime=250})
--]]

function PopOut(actor, adjustments)
	local expandTo		= GetParam(adjustments, "expandTo",		{1.2, 1.2})
	local expandTime	= GetParam(adjustments, "expandTime",	100)
	local shrinkTime	= GetParam(adjustments, "shrinkTime",	250)
	
	local originalScale = actor.extraScale
	
	ScaleTo(actor, expandTo, expandTime)
	ScriptDelay(expandTime,"DialogTimers")
	
	ScaleTo(actor, {0.0, 0,0}, shrinkTime)
	ScriptDelay(shrinkTime,"DialogTimers")
	
	actor.visible = false

	actor.extraScale = originalScale
end


--[[
	IsScaling
	
	Determines if an actor is actively scaling (over time.)
	
	Params:
		actor: The actor to query.
		
	Returns:
		true if the actor is actively scaling, false otherwise.
--]]

function IsScaling(actor)
	return actor.scaler and actor.scaler:IsRunning()
end


--[[
	WaitForScalingCompletion

	Waits for an actor to finish scaling.
	This method will fail unless it is run in a coroutine-based script.
	
	Params:
		actor: The actor to wait for.
		
	Returns:
		nil
--]]

function WaitForScalingCompletion(actor)
	while IsScaling(actor) do
		ScriptBreak()
	end
end


--[[
	Pop
	
	Doesn't touch current scale. Sets its "visible" property to true, scales it
	in over time, usually to a size slightly larger than normal.  Finally it scales the actor to
	its normal size.
	
	This function will not return until all scaling is complete.  Furthermore, it cannot be called
	outside of a coroutine-based script.
	
	Params:
		actor: The actor to scale.
		adjustments: A table of values adjusting aspects of the animation.  These include:
			expandTo: The amount to initially scale to.
			expandTime: How long the initial expansion should take.
			shrinkTime: How long the final shrink (to normal) should take.
			
	Returns:
		nil
			
	Examples:
		-- Pops myActor in using default values.
		Pop(actor)
		
		-- Pops myActor in, modifying the amount it should initially scale to.
		Pop(actor, {expandTo={1.5, 0.25})
		
		-- Pops myActor in, modifying the amount of time it should take to expand and shrink the
		-- actor.
		Pop(actor, {expandTime=500, shrinkTime=250})
--]]

function NearlyEqual(a,b)
	local difference = a - b
	return math.abs(difference) < 0.0001
end

function Pop(actor, adjustments)
	if(not NearlyEqual(actor.extraScale.x,1) or not NearlyEqual(actor.extraScale.y,1)) then
		logprint("Warning: Pop called on actor " .. actor.name .. " with extraScale of " .. actor.extraScale.x .. "," .. actor.extraScale.y .. ". Use PopKeepScale instead.")
	end
	local expandTo		= GetParam(adjustments, "expandTo",		{1.2, 1.2})
	local expandTime	= GetParam(adjustments, "expandTime",	250)
	local shrinkTime	= GetParam(adjustments, "shrinkTime",	100)
	local finalScale	= GetParam(adjustments, "finalScale",	{1.0, 1.0})
	
	actor.visible = true

	ScaleTo(actor, expandTo, expandTime)
	ScriptDelay(expandTime,"DialogTimers")
	
	ScaleTo(actor, finalScale, shrinkTime)
	ScriptDelay(shrinkTime,"DialogTimers")
end

--The above calls Actor.visible = true, which is a bug (see FogBugz 275), but I don't want to break Blokus. A fixed version is listed below. Once Blokus is branched, the versions should be merged. --JSUPNIK

function CorrectedPop(actor, adjustments)
	if(not NearlyEqual(actor.extraScale.x,1) or not NearlyEqual(actor.extraScale.y,1)) then
		logprint("Warning: CorrectPop called on actor " .. actor.name .. " with extraScale of " .. actor.extraScale.x .. "," .. actor.extraScale.y .. ". Use PopKeepScale instead.")
	end
	local expandTo		= GetParam(adjustments, "expandTo",		{1.2, 1.2})
	local expandTime	= GetParam(adjustments, "expandTime",	250)
	local shrinkTime	= GetParam(adjustments, "shrinkTime",	100)
	local finalScale	= GetParam(adjustments, "finalScale",	{1.0, 1.0})

	ScaleTo(actor, expandTo, expandTime)
	ScriptDelay(expandTime,"DialogTimers")
	
	ScaleTo(actor, finalScale, shrinkTime)
	ScriptDelay(shrinkTime,"DialogTimers")
end

--The above function plays well with actors that already have extraScale. In particular, it does its scaling relative to existing extraScale, and also waits for any existing scaling functions to end
function PopKeepScale(actor, adjustments)
	local expandTo		= GetParam(adjustments, "expandTo",		{1.2, 1.2})
	local expandTime	= GetParam(adjustments, "expandTime",	250)
	local shrinkTime	= GetParam(adjustments, "shrinkTime",	100)

	--Coax dimensions to an X,Y pattern
	if(expandTo[1] ~= nil) then
		expandTo = {x = expandTo[1], y = expandTo[2]}
	end

	while(IsScaling(actor)) do
		ScriptBreak()
	end
	local oldScale = actor.extraScale
	ScaleTo(actor,{expandTo.x * oldScale.x,expandTo.y * oldScale.y},expandTime)
	ScriptDelay(expandTime,"DialogTimers")
	ScaleTo(actor,oldScale,shrinkTime)
	ScriptDelay(shrinkTime,"DialogTimers")
end

function StopScaling(actor)
	if(actor.scaler ~= NIL) then
		actor.scaler:Stop()
	end
end