
--[[
	StartScoreFlash
	
	Starts a score flash on the current screen.  Score flashes are placed onto the screen's
	score flash layer, which must be a child of the screen and must be named, "scoreFlashLayer".
	
	Params:
		params: A table of parameters.  See the "Named Params" section for more info.
		
	Named Params:
		alignmentTarget: An instance of Actor which, if specified, will cause the score flash to be
			positioned relative to the actor's global rect.
		alignmentPoint: Specifies where in the alignmentTarget that the score flash should be
			positioned.  Defaults to "Center".
		offset: Offsets the position of the score flash by a given amount.
		position: Specifies where to place the score flash.  This value will be ignored if
			alignmentTarget is set.\
		color: Specifies what color to override the text color of the scoreflash with
		resource: The resource that describes the score flash.  This can either be a table, or a
			string.  If it's a string, then that string must point to a resource in the global
			table, "Resources".  If neither are specified, "DefaultScoreFlash" will be used.
	
	Returns:
		nil
--]]

function StartScoreFlash(params)
	-- Extract params, using a reasonable set of defaults.
	params = params or {}
	local resource = params.resource or "DefaultScoreFlash"
	local text = params.text or ""
	local color = params.color or ""
	
	-- Perform additional processing on params.
	if type(resource) == "string" then
		resource = Resources[resource]
		if not resource then
			error("StartScoreFlash could not find the resource, \""..tostring(resource).."\".")
		end
	end
	
	-- Find a point to align the score flash to.
	local position = GetGlobalAlignmentPoint(params)

	-- Find a container to place the score flash in.
	local screen = Screens.currentScreen
	local scoreFlashLayer = screen.scoreFlashLayer

	-- Create the score flash
	local scoreFlash = CreateInstance(resource)
	
	-- Setup text on the score flash
	scoreFlash.text = text

	if(color ~= nil) then
		scoreFlash.fgColor = color
	end
	
	-- Position the score flash
	scoreFlash.center = position
	
	-- If scoreflash goes off the screen, move onto screen
	if(scoreFlash.x + scoreFlash.width > screen.width) then
		local delta = (scoreFlash.x + scoreFlash.width) - screen.width
		scoreFlash.position = {scoreFlash.position.x - delta,scoreFlash.position.y}
	end
	if(scoreFlash.x < 0) then
		scoreFlash.position = {0,scoreFlash.position.y}
	end
	if(scoreFlash.y + scoreFlash.height > screen.height) then
		local delta = (scoreFlash.y + scoreFlash.height) - screen.height
		scoreFlash.position = {scoreFlash.position.x,scoreFlash.position.y - delta}
	end
	if(scoreFlash.y < 0) then
		scoreFlash.position = {scoreFlash.position.x,0}
	end
		
	-- Add the score flash to the container
	scoreFlashLayer:AddChild(scoreFlash)
end

--[[
	StartScoreFlash
	
	Starts a score flash on the current screen.  Score flashes are placed onto the screen's
	score flash layer, which must be a child of the screen and must be named, "scoreFlashLayer".
	
	Params:
		params: A table of parameters.  See the "Named Params" section for more info.
		
	Named Params:
		alignmentTarget: An instance of Actor which, if specified, will cause the score flash to be
			positioned relative to the actor's global rect.
		alignmentPoint: Specifies where in the alignmentTarget that the score flash should be
			positioned.  Defaults to "Center".
		offset: Offsets the position of the score flash by a given amount.
		position: Specifies where to place the score flash.  This value will be ignored if
			alignmentTarget is set.\
		color: Specifies what color to override the text color of the scoreflash with
		resource: The resource that describes the score flash.  This can either be a table, or a
			string.  If it's a string, then that string must point to a resource in the global
			table, "Resources".  If neither are specified, "DefaultScoreFlash" will be used.
	
	Returns:
		nil
--]]

function StartBitmapScoreFlash(params)
	-- Extract params, using a reasonable set of defaults.
	params = params or {}
	local resource = params.resource or "DefaultScoreFlash"
	local text = params.text or ""
	local color = params.color or ""
	
	-- Perform additional processing on params.
	if type(resource) == "string" then
		resource = Resources[resource]
		if not resource then
			error("StartScoreFlash could not find the resource, \""..tostring(resource).."\".")
		end
	end
	
	-- Find a point to align the score flash to.
	local position = GetGlobalAlignmentPoint(params)

	-- Find a container to place the score flash in.
	local screen = Screens.currentScreen
	local scoreFlashLayer = screen.scoreFlashLayer

	-- Create the score flash
	local scoreFlash = CreateInstance(resource)
	
	-- Setup text on the score flash
	scoreFlash.text = text

	if(color ~= nil) then
		scoreFlash.color = color
	end
	
	-- Position the score flash
	scoreFlash.center = position
	
	-- If scoreflash goes off the screen, move onto screen
	if(scoreFlash.x + scoreFlash.width > screen.width) then
		local delta = (scoreFlash.x + scoreFlash.width) - screen.width
		scoreFlash.position = {scoreFlash.position.x - delta,scoreFlash.position.y}
	end
	if(scoreFlash.x < 0) then
		scoreFlash.position = {0,scoreFlash.position.y}
	end
	if(scoreFlash.y + scoreFlash.height > screen.height) then
		local delta = (scoreFlash.y + scoreFlash.height) - screen.height
		scoreFlash.position = {scoreFlash.position.x,scoreFlash.position.y - delta}
	end
	if(scoreFlash.y < 0) then
		scoreFlash.position = {scoreFlash.position.x,0}
	end
		
	-- Add the score flash to the container
	scoreFlashLayer:AddChild(scoreFlash)
end

Resources.DefaultScoreFlash = Resources.Actor:new {
	class = "Label",
	layer = 2,
	fgColor = "LightYellow",
	font = Resources.MainFontMedium:new {
		resource = "MainFontMedium",
	},
	scripts = {"ScoreFlashScript"},
	shaderType = SHADERTYPE_BLENDCOMBINEDALPHA,
}

Resources.DefaultGraphicalScoreFlash = Resources.Actor:new {
	class = "Actor",
	layer = 2,
	scripts = {"ScoreFlashScript"},
	shaderType = SHADERTYPE_BLENDCOMBINEDALPHA,
}

Scripts.ScoreFlashScript = function(scoreFlash) 
	ScriptDelay(200)
	FadeOut(scoreFlash,600)
	local dest = scoreFlash.position
	dest.y = dest.y - 50
	if(dest.y < 0) then
		dest.y = 0
	end
	SlideAndWait(scoreFlash,dest,600)
	scoreFlash:Destroy();
end

--[[
	StartScoreFlash
	
	Starts a score flash on the current screen.  Score flashes are placed onto the screen's
	score flash layer, which must be a child of the screen and must be named, "scoreFlashLayer".
	
	Params:
		params: A table of parameters.  See the "Named Params" section for more info.
		
	Named Params:
		alignmentTarget: An instance of Actor which, if specified, will cause the score flash to be
			positioned relative to the actor's global rect.
		alignmentPoint: Specifies where in the alignmentTarget that the score flash should be
			positioned.  Defaults to "Center".
		offset: Offsets the position of the score flash by a given amount.
		position: Specifies where to place the score flash.  This value will be ignored if
			alignmentTarget is set.
		resource: The resource that describes the score flash.  This can either be a table, or a
			string.  If it's a string, then that string must point to a resource in the global
			table, "Resources".  If neither are specified, "DefaultScoreFlash" will be used.
	
	Returns:
		nil
--]]

function StartGraphicalScoreFlash(params)
	-- Extract params, using a reasonable set of defaults.
	params = params or {}
	local resource = params.resource or "DefaultGraphicalScoreFlash"
	local image = params.image or ""
	
	-- Perform additional processing on params.
	if type(resource) == "string" then
		resource = Resources[resource]
		if not resource then
			error("StartScoreFlash could not find the resource, \""..tostring(resource).."\".")
		end
	end
	
	-- Find a point to align the score flash to.
	local position = GetGlobalAlignmentPoint(params)

	-- Find a container to place the score flash in.
	local screen = Screens.currentScreen
	local scoreFlashLayer = screen.scoreFlashLayer

	-- Create the score flash
	local scoreFlash = CreateInstance(resource)
	
	-- Setup text on the score flash
	scoreFlash.image = image
	
	-- Position the score flash
	scoreFlash.center = position
	
	-- If scoreflash goes off the screen, move onto screen
	if(scoreFlash.x + scoreFlash.width > screen.width) then
		local delta = (scoreFlash.x + scoreFlash.width) - screen.width
		scoreFlash.position = {scoreFlash.position.x - delta,scoreFlash.position.y}
	end
	if(scoreFlash.x < 0) then
		scoreFlash.position = {0,scoreFlash.position.y}
	end
	if(scoreFlash.y + scoreFlash.height > screen.height) then
		local delta =  (scoreFlash.y + scoreFlash.height) - screen.height
		scoreFlash.position = {scoreFlash.position.x,scoreFlash.position.y - delta}
	end
	if(scoreFlash.y < 0) then
		scoreFlash.position = {scoreFlash.position.x,0}
	end
		
	-- Add the score flash to the container
	scoreFlashLayer:AddChild(scoreFlash)
end