
TICK_INTERVAL = 32

-- Create the "Scripts" table, where named scripts are to be stored.
Scripts = {}

-- Create a "Script" object.  This holds one entry, "current", which will store a reference to the
-- actively running instance of Script, or nil if one is not running.
--
-- NOTE, davidl: "current"'s entry has been set up in a table ("Script") for purposes of forward
-- compatibility.  The thought is that each Guru class will have a global variable in Lua,
-- i.e. "Actor" would be a global variable, pointing to class-specific information (and a
-- constructor), "Screen" would have one, so would "Button", "TextBox", and "Script".  Each class
-- will be able to have properties attached to it.  In the case of the Script class, it will have
-- a "current" property attached to it.
Script = {}


--[[
	ScriptBreak
	
	Causes the currently running Script to cease execution until it is run again (usually in the
	next Tick.)
	
	Params:
		nil
		
	Returns:
		nil
--]]

function ScriptBreak()
	assert(Script.current, "ScriptBreak() must be called from within a running script.")
	if coroutine.running() then
		coroutine.yield()
	end
end


--[[
	ScriptDelay

	Causes the currently running Script to delay execution for a given amount of time, which is
	measured in milliseconds.
	
	Params:
		delayInMS: The amount of time to delay the script for, measured in milliseconds.
		timerGroup: (default = "default") The timer group to use.  This is passed into
			TimerManager::StartTimer.
	
	Returns:
		nil
--]]

function ScriptDelay(delayInMS, timerGroup)
	assert(Script.current, "ScriptDelay() must be called from within a running script.")
	Script.current:Delay(delayInMS, timerGroup)
end


--[[
	IndicatesWaiting
	
	Helps to determine if a value is indicating that a function should wait until some arbitrary
	operation finishes.
	
	Params:
		value: A value, consisting of either a boolean value, nil, or one of the following two
			strings: "wait" or "nowait".  If it is not one of these values, an error will be
			generated (via Lua's built-in function, error().)
	
	Returns:
		true if 'value' is either true, or if it's the string, "wait"; false if 'value' is either
			false, nil, or the string, "wait".
--]]

function IndicatesWaiting(value)
	if type(value) == 'nil' then
		return false
	elseif type(value) == 'boolean' then
		return value
	elseif type(value) == 'string' then
		if value == "wait" then
			return true
		elseif value == "nowait" then
			return false
		else
			error(string.format("IndicatesWaiting(value) could not determine whether or not its caller should wait, as it was passed an invalid string value, \"%s\".",
				value))
		end
	else
		error(string.format("IndicatesWaiting(value) could not determine whether or not its caller should wait, as it was passed an invalid value of type, '%s'.",
			type(value)))
	end
end

