
--[[
	GetParam
	
	GetParam helps functions that want to take in a list of named arguments.  Given a possibly nil
	table of parameters and the name of the desired parameter, GetParam will either return a value
	from the table, or if one cannot be retrieved, a default value.
	
	Params:
		paramTable: A table of parameters, or nil.
		paramName: The name of the parameter.
		defaultValue: The default value.
		
	Returns:
		paramTable[paramName], if paramTable and paramTable[paramName] are not nil.  defaultValue
		is returned otherwise.
		
	Examples:
		-- Sets x to 123
		local f = function (paramTable)
			return GetParam(paramTable, "x", 123)
		end
		x = f()
		
		-- Sets x to 123
		local f = function (paramTable)
			return GetParam(paramTable, "x", 123)
		end
		x = f({y = 999})
		
		-- Sets x to 876
		local f = function (paramTable)
			return GetParam(paramTable, "x", 123)
		end
		x = f({x = 876})
--]]

function GetParam(paramTable, paramName, defaultValue)
	if not paramTable then
		return defaultValue
	else
		return paramTable[paramName] or defaultValue
	end
end


--[[
	Layout
	
	Picks a UI-layout-altering value given the current platform, and given
	various app-settings.
	
	Examples:
		myActor.alignmentOffset = Layout {
			{0, 25},			-- the default value
			iphone = {0, 10},	-- the value to use on iPhone and iPod Touch
			iphone5 = {0, 12},	-- the value to use on iPhone 5
			ipad = {0, 20},		-- the value to use on iPad
			pc = {0, 22},		-- the value to use on Mac and Windows PCs
		}
		
		-- use one alignment offset on iPhone 5, and another on all other platforms
		myOtherActor.alignmentOffset = Layout {
			{0, 0},				-- default value
			iphone5 = {0, 10},	-- iPhone 5 specific value
		}
--]]
function Layout(params)
	if IsAndroid() or (APP.useAndroidAssets) then
		if(IsAndroidShort())then   -- For short sized Android tablets
			if params.androidShort then
				return params.androidShort
			elseif params.android then
				return params.android
			elseif params.ipad then
				return params.ipad
			end
		else  -- Regular android
			if params.android then
				return params.android
			elseif params.iphone5 then
				return params.iphone5
			end
		end
	end

	if IsIPhone5() or APP.useIPhone5AppAssets then
		if params.iphone5 then
			return params.iphone5
		end
	end

    if IsIPhone4() or (APP.useIPhoneAppAssets and not (APP.useIPadAppAssets or APP.useIPhone5AppAssets)) then
        if params.iphone4 then
            return params.iphone4
        end
    end
	
	if IsIPhone() or APP.useIPhoneAppAssets then
		if params.iphone then
			return params.iphone
		end
	end
	
	if IsIPad() or APP.useIPadAppAssets then
		if IsIPadHalfSizingActive() and params.ipad_half then
			return params.ipad_half
		end
		if params.ipad then
			return params.ipad
		end
	end
	
	if (IsMac() or IsWindows()) and
		not (APP.useIPhone5AppAssets or APP.useIPhoneAppAssets or APP.useIPadAppAssets)
	then
		if params.pc then
			return params.pc
		end
	end
	
	if Use2XAppAssets() then
		if params.scale2x then
			return params.scale2x
		end
	end

	return params[1]
end

--[[
	ConfigPick
	
	Picks a UI-layout-altering value given a particular config value
	
	Examples:
		myActor.alignmentOffset = ConfigPick {
			{0, 25},			-- the default value
			EnableSmallSpinButton = {0, 10},	-- the value to use when config.EnableSmallSpinButton is true
		}
--]]

function ConfigPick(params)
	-- TODO: Is there a way to have the key be read from the params "Config.params.EnableSmallSpinButton" so that
	-- that config can be not hard coded here

	if Config.EnableSmallSpinButton == true and params.EnableSmallSpinButton then
		return params.EnableSmallSpinButton
	end
	if Config.EnableCCLayout == true and params.EnableCCLayout then
		return params.EnableCCLayout
	end
	return params[1]
end

-- Counts the number of keys in a table
function GetTableCount(t)
	local count = 0
	for key,value in pairs(t) do
		count = count + 1
	end
	return count
end

-- Selects and returns a random element from an array.
function RandomArrayElement(t)
	-- Return nil if 't' is not a table, or has no numerically-indexed elements.
	if type(t) ~= 'table' or table.getn(t) == 0 then
		return
	end

	-- Select and return an item
	return t[math.random(1, table.getn(t))]
end

function Range(a, b)
    return function () return math.random(a, b) end
end


function Do_RangeF(a, b)
    local min, max

    if a > b then
        max = a
        min = b
    else
        max = b
        min = a
    end

    return (math.random() * (max - min)) + min
end

function Do_RandSelect(t)
	return t [ math.random(1,table.getn(t)) ]
end


function RandSelect(t)
    return function() return Do_RandSelect(t) end
end

function RangeF(a, b)
    return function() return Do_RangeF(a, b) end
end

function Round(a)
	return math.floor(a + 0.5)
end

function PrintRecursiveFromResourceKeys(object, resourceList, name, indent, done)
	done = done or {}
	indent = indent or 0
	local name = name or object.name
	
	-- make sure resource is valid
	if(resourceList == nil) then
		resourceList = GetFlattenedResourceList(object)
	elseif(type(resourceList) == "string" and Resources[resourceList]) then
		resourceList = Resources[resourceList]
	end
	
	if (resourceList == nil) then
		PrintRecursive(object, indent, objectDone);
		return; 
	end
	
	local ind = string.rep (" ", indent) -- indent it
	logprint(ind .. "[" .. tostring(name) .. "] = \n".. ind .."{");
	indent = indent + 4
	ind = string.rep (" ", indent) -- indent it
	if ((type(object) == "table" or type(object) == "userdata") and 
		(object.class or object.resources)) then
		for key in propertyIter(resourceList) do
			value = object[key]
			if(done[value]) then
				continue
			elseif((type(value) == "userdata" or type(value) == "table")) then
				done[value] = true
				local resourceList = value.resources or value.class or value
				local name = key or value[name]
				PrintRecursiveFromResourceKeys(value, resourceList, name, indent, done)
				logprint(ind..",\n");
			elseif(type(value) == "string" and not _G[value]) then
				logprint(ind .. "[" .. tostring(key) .. "] = \"" .. tostring(value) .. "\",")
			elseif(type(value) == "function") then
				logprint(ind .. "[" .. tostring(key) .. "] = " .. tostring(resourceList[key]) ..",")
			else
				logprint(ind .. "[" .. tostring(key) .. "] = " .. tostring(value) .. ",")
			end
		end
	elseif (type(object) == "table") then
		for key, value in pairs(object) do
			if (done[value]) then 
				continue
			elseif ((type(value) == "userdata" or type(value) == "table")) then
				done[value] = true
				local resourceList = value.resources or value.class or value
				local name = key or value[name]
				PrintRecursiveFromResourceKeys(value, resourceList, name, indent, done)
				logprint(ind..",\n");
			elseif(type(value) == "string" and not _G[value]) then
				logprint(ind .. "[" .. tostring(key) .. "] = \"" .. tostring(value) .. "\",")
			else
				logprint(ind .. "[" .. tostring(key) .. "] = " .. tostring(value) .. ",")
			end
		end
	end
	
	indent = indent - 4
	local ind = string.rep (" ", indent) -- indent it
	logprint(ind .."}");
end

function serialize(object)
	if object then
		local serialize = object.__serialize() or object.__tostring()
		return serialize(object)
	end
end

function SerializeUserData(object, resourceList, name, indent, done)
end

function PrintRecursive (object, indent, done)
	done = done or {}
	indent = indent or 0
	
	if((type(object) == "userdata" or type(object) == "table") and (object.resources or object.class)) then
		local resources = object.resources or object.class
		local name = object.name or object
		PrintRecursiveFromResourceKeys(object, resources, name, indent, done)
		return
	end
	
	local ind = string.rep (" ", indent) -- indent it
	logprint(ind .. "[" .. tostring(object.name) .. "] = \n".. ind .."{");
	indent = indent + 4
	ind = string.rep (" ", indent) -- indent it
	if(type(object) == "function") then object = object() end
	for key, value in pairs(object) do
		local ind = string.rep(" ", indent) -- indent it
		if type(value) == "table" and not done[value] then
			done[value] = true
			PrintRecursive(value, indent, done)
		elseif(type(value) == "string" and not _G[value]) then
			logprint(ind .. "[" .. tostring(key) .. "] = \"" .. tostring(value) .. "\",")
		else
			logprint(ind .. "[" .. tostring(key) .. "] = " .. tostring(value) .. ",")
		end
	end

	indent = indent - 4
	local ind = string.rep (" ", indent) -- indent it
	logprint(ind .."}");
end

function CopyTable( self, deep, ud_copy_fn )

    ud_copy_fn = ud_copy_fn or function ( ud ) return ud end

    local new_table = {}

    for key, value in pairs(self) do
        local new_key
        if ( deep and type( key ) == 'table') then
            new_key = CopyTable( key, deep, ud_copy_fn )
        elseif ( type( key ) == 'userdata' ) then
            new_key = ud_copy_fn( key )
        else
            new_key = key
        end

        local new_value
        if ( deep and type( value ) == 'table' ) then
            new_value = CopyTable( value, deep, ud_copy_fn )
        elseif ( type( value ) == 'userdata' ) then
            new_value = ud_copy_fn( value )
        else
            new_value = value
        end

        new_table[ new_key ] = new_value
    end

     return new_table
end

function CustomizeObject(object, customProps)
	local t = CopyTable(object, true)
	for k, v in pairs(customProps) do
		if k == "__index" then continue
		elseif type(v) == "table" then
			if t[k] ~= nil then
				if type(t[k]) == "table" then
					t[k] = CustomizeObject(t[k], v)
				else
					t[k] = CopyTable(v, true)
				end
			else
				t[k] = CopyTable(v, true)
			end
		else
			t[k] = v
		end
	end
end

function Dice(t)
	return math.random(1,t[1])
end

Print = function(value)
	logprint(tostring(value))	
end

--
-- Used by AS_Print() and AS_Print2().
--
local PrintActorInfo = function (actor, arg)
	for i=2,table.getn(arg) do
		if type(arg[i]) == 'string' then
			io.write(" ", arg[i], "=")
			local valueOfProp = actor[arg[i]]
			if type(valueOfProp) == 'string' then
				io.write("\"", tostring(valueOfProp), "\"")
			elseif type(valueOfProp) == 'table' then
				Serialize(valueOfProp)
			else
				io.write(tostring(valueOfProp))
			end
		end
	end
end

--
-- Writes information on a table of Actors to the current io stream (using io.write()).
--
-- param  1:    The table of actors.
-- params 2..N: The properties to retrieve and print.  If none are specified, "name" and "image"
--              are displayed.
--
-- See DebugUtils.h for more info on this function.
--
function AS_Print(...)
	if type(arg[1]) == 'nil' then
		return
	end
	
	if table.getn(arg) == 1 then
		table.insert(arg, "name")
		table.insert(arg, "image")
	end

    for k, v in pairs(arg[1]) do
        if type(v) == 'userdata' then
			io.write(tostring(k), ":")
			PrintActorInfo(v, arg)
			io.write("\n")
        end
    end
end

--
-- See DebugUtils.h for docs on AS_Print2().
--
function AS_Print2(...)
	if type(arg[1]) == 'nil' then
		return
	end
	
	if table.getn(arg) == 1 then
		table.insert(arg, "name")
		table.insert(arg, "image")
	end

    for k, v in pairs(arg[1]) do
        if type(v) == 'userdata' then
			io.write(tostring(k), ":")
			PrintActorInfo(v, arg)
			io.write("\n")
			
			local currentParent = v.parent
			local padding = "    "
			while type(currentParent) == 'userdata' do
				io.write(padding)
				PrintActorInfo(currentParent, arg)
				io.write("\n")
				currentParent = currentParent.parent
				padding = padding .. "  "
			end
        end
    end
end

function Modulus(product, divisor)
	return (product - math.floor(product/divisor)*divisor)
end

-- GetNames - returns hash table of the names in a table
-- t 
--
function GetNames(table)
	local names = {}
	for k,v in pairs(table) do
		names[k] = 1
	end
	return names
end

-- GetGlobalNames - returns hash table of current
--     global names
--
function GetGlobalNames()
	return GetNames(getfenv(1))
end

function PrintNames(table, recursive, indentLevel)
	recursive = recursive or false
	indentLevel = indentLevel or 0
	i = 0
	indent = "\t"
	while i < indentLevel do
		indent = indent .. "\t"
		i = i + 1
	end
	
	for k, v in pairs(table) do
		if type(v) == 'table' and recursive == true then
			PrintNames(v, true, indentLevel + 1)
		end
		print(indent, k, "\t\t", v)		
	end
end

function PrintGlobalNames(recursive)
	PrintNames(getfenv(1), recursive, 0)
end

-- DiffGlobalNames - shows diff of current global names
--     vs previously recorded
--
function DiffGlobalNames(table)
  local gtable = getfenv(1)
  local deleted = {}

  local added = {}
  for i,x in pairs(table) do
    if not gtable[i] then
      tinsert(deleted, i)
    end
  end
  for i,x in pairs(gtable) do
    if not table[i] then
      tinsert(added, i)
    end
  end
  sort(deleted)
  sort(added)
  print("Changes to global names:")
  print("    Deleted")
  for i = 1, getn(deleted) do
    print("        "..deleted[i])
  end
  print("    Added")
  for i = 1, getn(added) do
    print("        "..added[i])
  end
end

function DoCoResume(args)
	coroutine.resume(Script.current)
end

function errorhandler()
	return debug.traceback()
end

function erroroutput(error)
	logprint(error)
end

function DoLuaChunk(chunk)
	local success
	local err

	if type(chunk) == "string" then
		success, err = xpcall(loadstring(chunk), function(errorObj)
			logprint(errorObj)
			logprint(debug.traceback())
		end)
		if not success then 
			logprint(err) 
			return err
		end
	elseif type(chunk) == "function" then
		success, err = xpcall(chunk(), function(errorObj)
			logprint(errorObj)
			logprint(debug.traceback())
		end)
		if not success then 
			logprint(err) 
			return err
		end
	end
end

-- remove trailing and leading whitespace from string.
-- http://en.wikipedia.org/wiki/Trim_(8programming)
function trim(s)
  -- from PiL2 20.4
  return (s:gsub("^%s*(.-)%s*$", "%1"))
end

--[[
	BuildChildList
	
	Given an actor with children named with an increasing index that starts at 1, for example,
	"myChild1", "myChild2", "myChild3", etc., BuildChildList will create an array of these child
	actors.
	
	Params:
		parent: The parent actor.
		childNameFmt: Describes how the child actors are named, using '%d' in place of the index.
			For example, n the case of "myChild1", "myChild2", etc., this value would be,
			"myChild%d".
	
	Returns:
		An array of children.
--]]
function BuildChildList(parent, childNameFmt)
	local list = {}

	local i = 1
	while true do
		local btn = parent:FindChild(string.format(childNameFmt, i), true)
		if not btn then
			break
		else
			list[i] = btn
			i = i + 1
		end
	end
	
	return list
end

function JoinTables(...)
    local t = {}

    for k,v in ipairs(arg) do
        if type(v) == 'table' then
            for k2,v2 in pairs(v) do
                t[k2] = v2
            end
        end    
    end

    return t
end

function DeepCopy(t)
	if type(t) ~= 'table' then
		return t
	else
		local k, v, new_t
		new_t = {}
		for k, v in pairs(t) do
			new_t[k] = DeepCopy(v)
		end
		return new_t
	end
end

function DeepEquals(table1, table2)
    local avoid_loops = {}
    local function recurse(t1, t2)
        -- compare value types
        if type(t1) ~= type(t2) then return false end
        -- Base case: compare simple values
        if type(t1) ~= "table" then return t1 == t2 end
        -- Now, on to tables.
        -- First, let's avoid looping forever.
        if avoid_loops[t1] then return avoid_loops[t1] == t2 end
        avoid_loops[t1] = t2
        -- Copy keys from t2
        local t2keys = {}
        local t2tablekeys = {}
        for k, _ in pairs(t2) do
            if type(k) == "table" then table.insert(t2tablekeys, k) end
            t2keys[k] = true
        end
        -- Let's iterate keys from t1
        for k1, v1 in pairs(t1) do
            local v2 = t2[k1]
            if type(k1) == "table" then
                -- if key is a table, we need to find an equivalent one.
                local ok = false
                for i, tk in ipairs(t2tablekeys) do
                    if table_eq(k1, tk) and recurse(v1, t2[tk]) then
                        table.remove(t2tablekeys, i)
                        t2keys[tk] = nil
                        ok = true
                        break
                    end
                end
                if not ok then return false end
            else
                -- t1 has a key which t2 doesn't have, fail.
                if v2 == nil then return false end
                t2keys[k1] = nil
                if not recurse(v1, v2) then return false end
            end
        end
        -- if t2 has a key which t1 doesn't have, fail.
        if next(t2keys) then return false end
        return true
    end
    return recurse(table1, table2)
end

local function InvalidateTable_FormatKey(t, k)
	local str
	if IsString(k) then
		str = "[\"" .. tostring(k) .. "\"]"
	else
		str = "[" .. tostring(k) .. "]"
	end
	local parent = rawget(t, "_InvalidateTable_parent")
	if IsTable(parent) then
		local parentKey = rawget(t, "_InvalidateTable_key")
		return InvalidateTable_FormatKey(parent, parentKey) .. str
	else
		return str
	end
end

local InvalidateTable_metatable = {
	__newindex = function (t, k, v)
		error(string.format("Unable to set value in an invalidated Lua table!   (old table location:%s;  new value:<%20s...>", InvalidateTable_FormatKey(t, k), tostring(v)))
		--rawset(t, k, v)
	end,

	__index = function (t, k)
		error(string.format("Unable to get value from an invalidated Lua table! (old table location:%s;  new value:<%20s...>", InvalidateTable_FormatKey(t, k), tostring(v)))
		--return rawget(t, k)
	end,
}

function InvalidateTable(t, _activelyInvalidating)
	assert(IsTable(t))
	_activelyInvalidating = _activelyInvalidating or {}
	if _activelyInvalidating[t] == true then
		return
	end
	_activelyInvalidating[t] = true
	for _k, _v in pairs(t) do
		if IsTable(_v) then
			InvalidateTable(_v, _activelyInvalidating)
			rawset(_v, "_InvalidateTable_key", _k)
			rawset(_v, "_InvalidateTable_parent", t)
		end
		rawset(t, _k, nil)
	end
	setmetatable(t, InvalidateTable_metatable)
	_activelyInvalidating[t] = nil
end

function MergeTables(...) 
    local t = {}
    for n = 1,select("#",...) do
        local arg = select(n,...)
        if type(arg)=="table" then
            for _,v in ipairs(arg) do
                t[#t+1] = v
            end
        else
            t[#t+1] = arg
        end
    end
    return t
end

local table_typename = "table"
function IsTable(v)
	return type(v) == table_typename
end

local nil_typename = "nil"
function IsNil(v)
	return type(v) == nil_typename
end

local number_typename = "number"
function IsNumber(v)
	return type(v) == number_typename
end

local string_typename = "string"
function IsString(v)
	return type(v) == string_typename
end

local function_typename = "function"
function IsFunction(v)
	return type(v) == function_typename
end

local userdata_typename = "userdata"
function IsUserdata(v)
	return type(v) == userdata_typename
end
function IsUserData(v)
	return type(v) == userdata_typename
end

local boolean_typename = "boolean"
function IsBoolean(v)
	return type(v) == boolean_typename
end

--Fisher-Yates shuffle. Sample code from Wikipedia.
function table.shuffle(t)
  local n = #t -- n is the last pertinent index (loop invariant)
  while n > 1 do
    local k = math.random(n) -- 1 <= k <= n
    -- Quick swap
    t[n], t[k] = t[k], t[n]
    n = n - 1
  end
 
  return t
end


--[[
	table.insertall(t, srcElems)

	Copies all elements of one table into another

	Integer-keyed elements that are consecutively indexed from 1 to N, will be inserted
	at the end of 't', via table.insert, such that existing elements in 't' will *NOT*
	be overwritten.

	Other elements, such as string-keyed elements, or integers whose keys are
	non-consecutively indexed from 1 to N, will be copied into 't' using the [] operator,
	such that existing elements in 't' *WILL* be overwritten!  (I.e. table.insertall()
	doesn't make any attempts to rename/reindex non-integer keys, such as strings.)

	Params:
		t: the table to insert elements into
		srcElems: a table to copy elements from.  If 'srcElems' isn't a table,
			then it will be inserted directly into 't', via table.insert(t, srcElems)

	Returns:
		nil
--]]
function table.insertall(t, srcElems)
	if IsTable(srcElems) then
		-- Make an indexed list of all keys
		local allKeys = {}
		for k, v in pairs(srcElems) do
			allKeys[k] = true
		end
		-- Insert consecutively-indexed, integer-keyed values into 't'
		for i, v in ipairs(srcElems) do
			--print("--- insert:", i, v)
			table.insert(t, v)
			allKeys[i] = false
		end
		-- Insert the rest of the values into 't'
		for k, v in pairs(srcElems) do
			if allKeys[k] then
				--print("--- copy:", k, v)
				t[k] = v
			end
		end
	else
		-- Non-tables will just get inserted directly into t.
		table.insert(t, srcElems)
	end
end


--[[
	FindTableValue
	
	FindTableValue searches a table for a value.  Searches are not recursive.
	The table is iterated through via 'pairs()'.  Equality is tested using the
	expression, 'query == iteratedValue', where 'query' is the value to look for
	and 'iteratedValue' is a value that exists in the table.
	
	Params:
		t: The table to search through.
		query: The value to look for.
		
	Returns:
		The key that the queried-for value can be found at.  If such a value
		does not exist, then nil will be returned.
--]]

function FindTableValue(t, query)
	for key, iteratedValue in pairs(t) do
		if query == iteratedValue then
			return key
		end
	end
end


--[[
	StringEndsWith
	
	Given two strings, this will return 'true' if the first string has the second
	string as a suffix.  If not, 'false' will be returned.
	
	Params:
		str: The string to check against
		suffix: The potential suffix
	
	Returns:
		Boolean true if 'str' ends with the given suffix, false otherwise.
--]]
function StringEndsWith(str, suffix)
	return (suffix == "") or (string.sub(str, -string.len(suffix)) == suffix)
end


-- Shorthand notation for console debugging functions:
lai = LogActorInfo
gso = GetScriptObject
ladc = LogAPPDataCache
larpc = LogAPPRemotePlayerCache

-- Additional console debugging functions, in shorthand form.
nud = function (actor, x, y)	-- Nudge Actor
	actor = GetScriptObject(actor)
	if not actor then return end
	if actor.alignment == "NoAlignment" then
		actor.position = {actor.x + x, actor.y + y}
	else
		actor.alignmentOffset = {actor.alignmentOffset.x + x, actor.alignmentOffset.y + y}
	end
end

local LogActorInfo_AlignmentProperties = {
	{'pos',		'position'},
	{'size',	'size'},
	{'ali',		'alignment'},
	{'aTo',		'alignTo'},
	{'aOff',	'alignmentOffset'},
	{'lVis',	'visible'},
	{'z',		'zorder'},
}

laia = function (...)			-- Log Actor Info, along with alignment properties
	return lai(..., LogActorInfo_AlignmentProperties)
end

--Bitwise operators (These are implemented in LUA 5.2)
--Implementation from: http://ricilake.blogspot.com/2007/10/iterating-bits-in-lua.html

function Bit(p) 
	return 2 ^ (p - 1) -- 1-based indexing 
end 

-- Typical call: if HasBit(x, Bit(3)) then ... 
function HasBit(x, p) 
	return x % (p + p) >= p 
end 

function SetBit(x, p) 
	return HasBit(x, p) and x or x + p 
end 

function ClearBit(x, p) 
	return HasBit(x, p) and x - p or x 
end

function MatchHeightToScreen(a)
	local backgroundHeight = a.size.y
	local s = APP.screenSize

	if(s)then
		local screenHeight = APP.screenSize.y
		
		--logprint("backgroundHeight = "..backgroundHeight)
		--logprint("screenHeight = "..screenHeight)
        
		--if(screenHeight>backgroundHeight)then
			
			a.size = {a.size.x,screenHeight}
		
		--end
	else
		logprint("************* MatchHeightToScreen - no CurrentScreen")
	end
end

--ExecuteInline : A wrapper function that calls another function (used for macros in level definitions)
--Parameters:
	-- f : a zero argument function that contains arbitrary code
function ExecuteInline(f)
	return f()
end